/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */

/* 
 * Apache example module.  Provide demonstrations of how modules do things.
 *
 */

#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "util_script.h"

#include <openssl/ssl.h>
#ifdef APACHE20
#include "mod_ssl.h"
#endif /* APACHE20 */

/*
 * Akenti based includes
 */

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <string>
#include <errno.h>

#include <set>
#include "Exception.hpp"
#include "StringTokenizer.hpp"
#include "AkentiEngine.hpp"
#include "CapabilityCertificate.hpp"
#include "CertificateFactory.hpp"
#include "Certificate.hpp"
#include "AkentiInit.hpp"
#include "AkentiPrincipal.hpp"
#include "AkentiMessage.hpp"

#include "URLConnection.hpp"
#include "URL.hpp"
#include "Exception.hpp"
#include "Error.hpp"

#include "CertificateRetrievalFactory.hpp"
#include "Certificate.hpp"
#include "Directory.hpp"
#include "Exception.hpp"

using namespace std;
#include "ap_compat.h"
#include "PrintInfo.hpp"
//#include "TestExtAuth.hpp"


using Akenti::URLConnection;
using Akenti::URL;
using Akenti::Exception;
using Akenti::Error;

/*
 * Forward declarations
 */
#ifdef APACHE20
extern "C" module AP_MODULE_DECLARE_DATA akenti_module;
#else /* APACHE20 */
extern "C" module MODULE_VAR_EXPORT akenti_module;
#endif /* APACHE20 */


/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Data declarations.                                                       */
/*                                                                          */
/* Here are the static cells and structure declarations private to our      */
/* module.                                                                  */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*
 * Sample configuration record.  Used for both per-directory and per-server
 * configuration data.
 *
 * It's perfectly reasonable to have two different structures for the two
 * different environments.  The same command handlers will be called for
 * both, though, so the handlers need to be able to tell them apart.  One
 * possibility is for both structures to start with an int which is zero for
 * one and 1 for the other.
 *
 * Note that while the per-directory and per-server configuration records are
 * available to most of the module handlers, they should be treated as
 * READ-ONLY by all except the command and merge handlers.  Sometimes handlers
 * are handed a record that applies to the current location by implication or
 * inheritance, and modifying it will change the rules for other locations.
 */
typedef struct akentiCfg {
    char *configfile;		/* config file for Akenti */
    char *akresources;
} akentiCfg;

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* The following pseudo-prototype declarations illustrate the parameters    */
/* passed to command handlers for the different types of directive          */
/* syntax.  If an argument was specified in the directive definition        */
/* (look for "command_rec" below), it's available to the command handler    */
/* via the (void *) info field in the cmd_parms argument passed to the      */
/* handler (cmd->info for the examples below).                              */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*
 * Command handler for a NO_ARGS directive.
 *
 * static const char *handle_NO_ARGS(cmd_parms *cmd, void *mconfig);
 */

/*
 * Command handler for a RAW_ARGS directive.  The "args" argument is the text
 * of the commandline following the directive itself.
 *
 * static const char *handle_RAW_ARGS(cmd_parms *cmd, void *mconfig,
 *                                    const char *args);
 */

/*
 * Command handler for a FLAG directive.  The single parameter is passed in
 * "bool", which is either zero or not for Off or On respectively.
 *
 * static const char *handle_FLAG(cmd_parms *cmd, void *mconfig, int bool);
 */

/*
 * Command handler for a TAKE1 directive.  The single parameter is passed in
 * "word1".
 *
 * static const char *handle_TAKE1(cmd_parms *cmd, void *mconfig,
 *                                 char *word1);
 */

/*
 * Command handler for a TAKE2 directive.  TAKE2 commands must always have
 * exactly two arguments.
 *
 * static const char *handle_TAKE2(cmd_parms *cmd, void *mconfig,
 *                                 char *word1, char *word2);
 */

/*
 * Command handler for a TAKE3 directive.  Like TAKE2, these must have exactly
 * three arguments, or the parser complains and doesn't bother calling us.
 *
 * static const char *handle_TAKE3(cmd_parms *cmd, void *mconfig,
 *                                 char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a TAKE12 directive.  These can take either one or two
 * arguments.
 * - word2 is a NULL pointer if no second argument was specified.
 *
 * static const char *handle_TAKE12(cmd_parms *cmd, void *mconfig,
 *                                  char *word1, char *word2);
 */

/*
 * Command handler for a TAKE123 directive.  A TAKE123 directive can be given,
 * as might be expected, one, two, or three arguments.
 * - word2 is a NULL pointer if no second argument was specified.
 * - word3 is a NULL pointer if no third argument was specified.
 *
 * static const char *handle_TAKE123(cmd_parms *cmd, void *mconfig,
 *                                   char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a TAKE13 directive.  Either one or three arguments are
 * permitted - no two-parameters-only syntax is allowed.
 * - word2 and word3 are NULL pointers if only one argument was specified.
 *
 * static const char *handle_TAKE13(cmd_parms *cmd, void *mconfig,
 *                                  char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a TAKE23 directive.  At least two and as many as three
 * arguments must be specified.
 * - word3 is a NULL pointer if no third argument was specified.
 *
 * static const char *handle_TAKE23(cmd_parms *cmd, void *mconfig,
 *                                  char *word1, char *word2, char *word3);
 */

/*
 * Command handler for a ITERATE directive.
 * - Handler is called once for each of n arguments given to the directive.
 * - word1 points to each argument in turn.
 *
 * static const char *handle_ITERATE(cmd_parms *cmd, void *mconfig,
 *                                   char *word1);
 */

/*
 * Command handler for a ITERATE2 directive.
 * - Handler is called once for each of the second and subsequent arguments
 *   given to the directive.
 * - word1 is the same for each call for a particular directive instance (the
 *   first argument).
 * - word2 points to each of the second and subsequent arguments in turn.
 *
 * static const char *handle_ITERATE2(cmd_parms *cmd, void *mconfig,
 *                                    char *word1, char *word2);
 */

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* These routines are strictly internal to this module, and support its     */
/* operation.  They are not referenced by any external portion of the       */
/* server.                                                                  */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* We prototyped the various syntax for command handlers (routines that     */
/* are called when the configuration parser detects a directive declared    */
/* by our module) earlier.  Now we actually declare a "real" routine that   */
/* will be invoked by the parser when our "real" directive is               */
/* encountered.                                                             */
/*                                                                          */
/* If a command handler encounters a problem processing the directive, it   */
/* signals this fact by returning a non-NULL pointer to a string            */
/* describing the problem.                                                  */
/*                                                                          */
/* The magic return value DECLINE_CMD is used to deal with directives       */
/* that might be declared by multiple modules.  If the command handler      */
/* returns NULL, the directive was processed; if it returns DECLINE_CMD,    */
/* the next module (if any) that declares the directive is given a chance   */
/* at it.  If it returns any other value, it's treated as the text of an    */
/* error message.                                                           */
/*                                                                          */
/*--------------------------------------------------------------------------*/

static const char *cmd_akconf(cmd_parms *cmd, void *mconfig,
			      char *config)
{
  akentiCfg *cfg = (akentiCfg *) mconfig;
  cfg->configfile =  config;

  return NULL;
}

static const char *cmd_akresources(cmd_parms *cmd, void *mconfig,
				   char *resources)
{
  akentiCfg *cfg = (akentiCfg *) mconfig;
  cfg->akresources = resources;

  return NULL;
}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Now we declare our content handlers, which are invoked when the server   */
/* encounters a document which our module is supposed to have a chance to   */
/* see.  (See mod_mime's SetHandler and AddHandler directives, and the      */
/* mod_info and mod_status examples, for more details.)                     */
/*                                                                          */
/* Since content handlers are dumping data directly into the connexion      */
/* (using the r*() routines, such as rputs() and rprintf()) without         */
/* intervention by other parts of the server, they need to make             */
/* sure any accumulated HTTP headers are sent first.  This is done by       */
/* calling send_http_header().  Otherwise, no header will be sent at all,   */
/* and the output sent to the client will actually be HTTP-uncompliant.     */
/*									    */
/* NOTE: No Content handlers used. The purpose of this module is to check   */
/*       if the client is authorized to view the content. Akenti has nothing*/
/*	 to do with handling the content and is to left to other modules    */
/*	 and the web server.						    */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Now let's declare routines for each of the callback phase in order.      */
/* (That's the order in which they're listed in the callback list, *not     */
/* the order in which the server calls them!  See the command_rec           */
/* declaration near the bottom of this file.)  Note that these may be       */
/* called for situations that don't relate primarily to our function - in   */
/* other words, the fixup handler shouldn't assume that the request has     */
/* to do with "example" stuff.                                              */
/*                                                                          */
/* With the exception of the content handler, all of our routines will be   */
/* called for each request, unless an earlier handler from another module   */
/* aborted the sequence.                                                    */
/*                                                                          */
/* Handlers that are declared as "int" can return the following:            */
/*                                                                          */
/*  OK          Handler accepted the request and did its thing with it.     */
/*  DECLINED    Handler took no action.                                     */
/*  HTTP_mumble Handler looked at request and found it wanting.             */
/*                                                                          */
/* What the server does after calling a module handler depends upon the     */
/* handler's return value.  In all cases, if the handler returns            */
/* DECLINED, the server will continue to the next module with an handler    */
/* for the current phase.  However, if the handler return a non-OK,         */
/* non-DECLINED status, the server aborts the request right there.  If      */
/* the handler returns OK, the server's next action is phase-specific;      */
/* see the individual handler comments below for details.                   */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*
 * This function gets called to create a per-directory configuration
 * record.  This will be called for the "default" server environment, and for
 * each directory for which the parser finds any of our directives applicable.
 * If a directory doesn't have any of our directives involved (i.e., they
 * aren't in the .htaccess file, or a <Location>, <Directory>, or related
 * block), this routine will *not* be called - the configuration for the
 * closest ancestor is used.
 *
 * The return value is a pointer to the created module-specific
 * structure.
 */
#ifdef APACHE20
static void *akenti_create_dir_config(apr_pool_t *p, char *dirspec)
#else /* APACHE20 */
static void *akenti_create_dir_config(pool *p, char *dirspec)
#endif /* APACHE20 */
{
    akentiCfg *cfg = (akentiCfg *) ap_pcalloc(p, sizeof(akentiCfg));
    cfg->configfile = NULL;
    return cfg;
}

static int auth_cert_forbid(request_rec *r, akentiCfg *cfg, char* message) {
  return HTTP_FORBIDDEN;
}

static int auth_cert_allow(request_rec *r, akentiCfg *cfg) {
  return OK; 
}

bool isDirectory(const char* FileName)
{
struct stat my_stat;
if (stat(FileName, &my_stat) != 0) return false;
return ((my_stat.st_mode & S_IFDIR) != 0);
}


/*
 * This routine is called to check for any module-specific restrictions placed
 * upon the requested resource.  (See the mod_access module for an example.)
 *
 * The return value is OK, DECLINED, or HTTP_mumble.  All modules with an
 * handler for this phase are called regardless of whether their predecessors
 * return OK or DECLINED.  The first one to return any other status, however,
 * will abort the sequence (and the request) as usual.
 *
 * This routine uses Akenti Policy engine to determine the access privileges of
 * the client's identity (X509) for a particular resource.
 *
 * NOTE:
 * ----
 * If Akenti engine controls only a sub-set of the resource tree, then it has
 * to decline the requests that do not have any ACL on them. For that reason,
 * the Apache server is configured to accept client certificates optionally.
 * This means, there may or may not be a peer for an SSL session.
 *
 * What if?
 * Say if akenti access check is called from a shared resource server, check for
 * SSL request. If no SSL, then check for the resource requested. If the
 * resource is not under akenti's control, then decline request. Otherwise,
 * forbid access.
 */

static int akenti_access_check(request_rec *r)
{

    int checkAccessCode = -1;
#ifdef SRS_DEBUG
    cerr <<  "SRS: In Akenti Module" << endl;
#endif /* SRS_DEBUG */

    akentiCfg *cfg = (akentiCfg *) ap_get_module_config(
				      r->per_dir_config,
				      &akenti_module);

    string uriRequest = r->uri;

    // special case : request for the root directory
    if (uriRequest == "/") {
      return DECLINED;
    }

    // check if the requested resource belong to the set
    // of akenti resources. If not, ignore the request
    StringTokenizer tokenizer(r->uri, "/");

    string resources = (cfg->akresources == NULL)?"":cfg->akresources;

    if (tokenizer.hasMoreTokens()) {
      string token = tokenizer.nextToken();
      string token1 = " " + token;
      string token2 = token + " ";

      if (!(resources.find(token1) != string::npos ||
	   (resources.find(token2) != string::npos) ||
	   (resources == token))) {
        return DECLINED;
      }
    }

    // Suppose if the request is non-SSL
    // An additional check is required for the resource requested.
    // Decline request if it is not under akenti's resource tree.
    // otherwise, forbid access.

#ifdef APACHE20
     SSLConnRec *sslconn = 
          (SSLConnRec *) ap_get_module_config (r->connection->conn_config, 
                                               &ssl_module);
     if (sslconn == NULL) {
#ifdef SRS_DEBUG
         cerr << "SRS: NULL sslconn" << endl;
#endif /* SRS_DEBUG */
         return auth_cert_forbid(r, cfg, "Expected Secure Connection");
    }
    SSL *ssl = (SSL *) sslconn->ssl;

    if (ssl == NULL) {
         return auth_cert_forbid(r, cfg, "Expected Secure Connection");
    }

   SSL_SESSION *session = NULL;
    if (ssl != NULL) {
       session = (SSL_SESSION *) SSL_get_session(ssl);
    }
#else /* APACHE20 */
    if (!ap_ctx_get(r->connection->client->ctx, "ssl")) {
      return auth_cert_forbid(r, cfg, "Expected Secure Connection");
    }

    SSL *ssl = (SSL *) ap_ctx_get(r->connection->client->ctx, "ssl");
    SSL_SESSION *session = (SSL_SESSION *) SSL_get_session(ssl);
#endif /* APACHE20 */

    if (session) {
#ifdef SRS_DEBUG
    cerr << "SRS: AkentiModule: Got session " << endl;
#endif /* SRS_DEBUG */
       X509 *peer = session->peer;

       // if there is a specific resource tree that akenti controls, then
       // an additional check is required to see if the resource is under
       // that tree or not. If yes, then return forbid, otherwise return
       // DECLINED (i.e. apache will act as if the akenti access check is
       //           not there)

       if (!peer) {
#ifdef SRS_DEBUG
    cerr << "SRS: AkentiModule: Did NOT get peer" << endl;
#endif /* SRS_DEBUG */
          return auth_cert_forbid(r, cfg, "Expected Peer Certificate");
       }

       string caname(1024, '\0');
       X509_NAME_oneline(X509_get_issuer_name(peer), 
	   		 (char *) caname.data(), 
			 caname.length());

       string subjectname(1024, '\0');
       X509_NAME_oneline(X509_get_subject_name(peer), 
                         (char *) subjectname.data(), 
	  	         subjectname.length());

       AkentiPrincipal user(subjectname.c_str(), caname.c_str());
       string status;

       if (!AkentiInit::init(cfg->configfile,status)) {
          return auth_cert_forbid(r, cfg, "Internal Engine Error");
       }
#ifdef SRS_DEBUG
    cerr << "SRS: Just past Akenti Init, subject = " << subjectname.c_str()  << endl;
#endif /* SRS_DEBUG */

       CapabilityCertificate cert;
       AkentiMessage msg;
       AkentiEngine engine;

       string resource = r->uri;

       // remove '/' from the beginning of the resource
       if (resource.length() > 0 && resource[0] == '/') {
         resource = resource.substr(1, resource.length()-1);
       }

       // remove '/' from the end of the resource
       if (resource.length() > 0 && resource[resource.length()-1] == '/') {
         resource = resource.substr(0, resource.length()-1);
       }
#ifdef SRS_DEBUG
       cerr << "SRS: 2.Akenti Module: resource = " << resource << endl;
       cerr << "SRS: 2.Akenti Module: filename = " << r->filename << endl;
#endif /* SRS_DEBUG */

// Remove last component if it is a file 
      if ((r->filename != NULL) && (!isDirectory (r->filename))) {
       // Remove last file component from string, so dir is left
       // This is because Akenti policy is directory based, not file based
          int i;
          string tmpresource = resource;
          for (i = tmpresource.length() - 1; i >= 0; i--) {
               if (tmpresource[i] == '/') break;
          }
          if (i > 0) {
              tmpresource = tmpresource.substr (0, i);
              resource = tmpresource;
          }
       }

       PrintInfo printInfo;
       //engine.setExternalEvaluator (new TestExtAuth);
       checkAccessCode = engine.checkAccess (resource, user, cert, msg);
#ifdef SRS_DEBUG 
       cerr << "SRS: Akenti Module: check access code = " << checkAccessCode << endl;
#endif /* SRS_DEBUG */
       if (checkAccessCode) {
           printInfo.getInfo (checkAccessCode, &cert, &msg);
           string actions = printInfo.getActions ();
#ifdef SRS_DEBUG 
           cerr << "SRS: mod_akenti: actions = " << actions << endl;
#endif /* SRS_DEBUG */
           if (actions != "NONE") {
#ifdef APACHE20
            apr_table_set (r->subprocess_env, "AKENTI_ACTIONS", actions.c_str());
#else /* APACHE20 */
              table_set (r->subprocess_env, "AKENTI_ACTIONS", actions.c_str());
#endif /* APACHE20 */
           }
 
           string conditionalInfo = printInfo.getConditionalInfo ();
#ifdef SRS_DEBUG 
           cerr << "SRS: mod_akenti: info = " << conditionalInfo << endl;
#endif /* SRS_DEBUG */
           if (conditionalInfo != "NONE") {
#ifdef APACHE20
              apr_table_set (r->subprocess_env, "AKENTI_CONDITIONAL_INFO",
                    conditionalInfo.c_str());
#else /* APACHE20 */
              table_set (r->subprocess_env, "AKENTI_CONDITIONAL_INFO",
                    conditionalInfo.c_str());
#endif /* APACHE20 */
           }
       }

       if (checkAccessCode) {
/* old code - ignore 
 *       if (engine.checkAccess(resource, user, cert, msg)) {
 */
         return auth_cert_allow(r, cfg);
       }
#ifdef SRS_DEBUG
    else {
       cerr << "SRS: Akenti Module: checkaccess failed ?" << endl;
    }
#endif /* SRS_DEBUG */
    } 
#ifdef SRS_DEBUG
    else {
       cerr << "SRS: Akenti Module: Failed to get session ?" << endl;
    }
#endif /* SRS_DEBUG */

    return auth_cert_forbid(r, cfg, "Request Rejected by Akenti Policy Engine");
}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* All of the routines have been declared now.  Here's the list of          */
/* directives specific to our module, and information about where they      */
/* may appear and how the command parser should pass them to us for         */
/* processing.  Note that care must be taken to ensure that there are NO    */
/* collisions of directive names between modules.                           */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * List of directives specific to our module.
 * NONE
 */

typedef const char *(*CMD_HAND_TYPE) ();
static const command_rec akenti_cmds[] =
{
    {
        "AkentiConf",              /* directive name */
        (CMD_HAND_TYPE)cmd_akconf,            /* config action routine */
        NULL,                   /* argument to include in call */
        RSRC_CONF,          /* per server */
	TAKE1,
        "Akenti Configuration File - 1 argument"
                                /* directive description */
    },
    {
        "AkentiResources",              /* directive name */
        (CMD_HAND_TYPE)cmd_akresources,            /* config action routine */
        NULL,                   /* argument to include in call */
        RSRC_CONF,          /* per server */
	TAKE1,
        "Akenti Resources (space delimiter) - 1 argument"
                                /* directive description */
    },
    {NULL}
};


/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Now the list of content handlers available from this module.             */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * List of content handlers our module supplies.  Each handler is defined by
 * two parts: a name by which it can be referenced (such as by
 * {Add,Set}Handler), and the actual routine name.  The list is terminated by
 * a NULL block, since it can be of variable length.
 *
 * Note that content-handlers are invoked on a most-specific to least-specific
 * basis; that is, a handler that is declared for "text/plain" will be
 * invoked before one that was declared for "text / *".  Note also that
 * if a content-handler returns anything except DECLINED, no other
 * content-handlers will be called.
 *
 * NONE
 */

#ifdef APACHE20
static void akenti_register_hooks (apr_pool_t *p)
{
    ap_hook_access_checker(akenti_access_check, NULL,NULL, APR_HOOK_MIDDLE);
}
#endif /* APACHE20 */

/*--------------------------------------------------------------------------*/
/*                                                                          */
/* Finally, the list of callback routines and data structures that          */
/* provide the hooks into our module from the other parts of the server.    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
/* 
 * Module definition for configuration.  If a particular callback is not
 * needed, replace its routine name below with the word NULL.
 *
 * The number in brackets indicates the order in which the routine is called
 * during request processing.  Note that not all routines are necessarily
 * called (such as if a resource doesn't have access restrictions).
 */

#ifdef APACHE20
extern "C" {
module AP_MODULE_DECLARE_DATA akenti_module =
{
    STANDARD20_MODULE_STUFF,
    akenti_create_dir_config,   /*per-directory config creator */
    NULL,   /* dir config merger */
    NULL,  /* server config creator */
    NULL,        /* server config merger */
    akenti_cmds,               /* command table */
    akenti_register_hooks     /* register hooks */
};
};
#else /* APACHE20 */
extern "C" {
module MODULE_VAR_EXPORT akenti_module =
{
    STANDARD_MODULE_STUFF,
    NULL,               /* module initializer */
    akenti_create_dir_config,	/*per-directory config creator */
    NULL,   /* dir config merger */
    NULL,  /* server config creator */
    NULL,        /* server config merger */
    akenti_cmds,               /* command table */
    NULL,           /* [9] list of handlers */
    NULL,  /* [2] filename-to-URI translation */
    NULL,      /* [5] check/validate user_id */
    NULL,       /* [6] check user_id is valid *here* */
    akenti_access_check,     /* [4] check access by host address */
    NULL,       /* [7] MIME type checker/setter */
    NULL,        /* [8] fixups */
    NULL,             /* [10] logger */
    NULL,      /* [3] header parser */
    NULL,         /* process initializer */
    NULL,         /* process exit/cleanup */
    NULL   /* [1] post read_request handling */
};
};
#endif /* APACHE20 */
