#include "UtcTime.hpp"
#include <iostream>
#include "Provider.hpp"
#include "OpenSSLProviderImpl.hpp"
#include "AkentiInit.hpp"
#include "AkentiEngine.hpp"
#include "AkentiMessage.hpp"
#include "AkentiPrincipal.hpp"
#include "CapabilityCertificate.hpp"
#include <vector>

extern "C" void doAccessCheck(
  char *confFile, char *dn,  char *ca, 
  char *resource, char *rights) {

  try {

    Provider po = Provider(new OpenSSLProviderImpl);
    Provider::setProvider(po);

    string status;

    if (!AkentiInit::init(confFile, status)) {
      rights = "CONFIG ERR\0";
      return;
    }

  } catch (exception& ex) {
    rights = "PROVIDER ERR\0";
    return;
  }

  Akenti::AkentiPrincipal principal(dn, ca);
  Akenti::AkentiEngine eng;
  Akenti::AkentiMessage msg;

  Akenti::CapabilityCertificate cert;
 
  if (eng.checkAccess(resource, principal, cert, msg)) {
    const vector<string>& actions = cert->getUnConditionalActions(); 

    string actionlist;

    int size = actions.size();

    if (size > 0) {
      actionlist = actions[0];
    }

    for (int i=1; i < size; i++) {
      actionlist += ",";
      actionlist += actions[i];
    }

    actionlist.copy(rights, actionlist.length(), 0);
    rights[actionlist.length()] = '\0';

  } else {
    rights[0] = '\0';
  }

  return;
}
