#ifndef CACHE_CERTIFICATE_IMPL_HPP
#define CACHE_CERTIFICATE_IMPL_HPP

#include <vector>
#include "AKCertHolder.hpp"

using std::vector;

namespace Akenti {

/**
  * The CacheCertificate holds one or more AkentiCertificates to be
  * cached. This certificate is used internally by 
  * the engine.
  *
  * @see AKCertHolderImpl
  * @see AkentiCertificateImpl
  * @see AkentiCertificate
  *
  * @version 1.1 00/05/26
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari 
  * @author Xiang Sun
  */

class CacheCertificateImpl : public AkentiCertificateImpl {

public:

  /**
    * Default constructor.
    */
  CacheCertificateImpl();

  /**
    * Default destructor.
    */
  ~CacheCertificateImpl();

  /**
    */
  string getHashString() const;

  /**
    */
  const vector<AKCertHolder>& getAKCertHolders() const;

protected:

  /**
    */
  void doWrite(AkentiOutputStream& drain) const;

  /**
    */
  void doRead(AkentiInputStream& source);

  /**
    */
  CacheCertificateImpl* newInstance() const;

  /**
    */
  void doParamString(ostream& os) const;

  void doToText(int indentAmt, ostream& os) const;

private:

  friend class CacheCertGenerator;

  /**
    * Populates this certificate with certificate
    * holders.
    */
  void setAKCertHolders(const vector<AKCertHolder>& holders);

  vector<AKCertHolder> holders;

};

} // end of name space

#endif
