#include "AkentiInputStream.hpp"
#include "StreamTokenizer.hpp"
#include "AkentiObject.hpp"
#include "DistinguishedName.hpp"
#include "UtcTime.hpp"
#include "URL.hpp"
#include "IOException.hpp" 
#include "Error.hpp" 

namespace Akenti {

typedef AkentiInputStream AIS;


AIS::AkentiInputStream(istream& is) : is(is) {
  if (!is) {
    throw Error("AkentiInputStream:constructor:bad input stream");
  }
}

AIS::~AkentiInputStream() {
}

AkentiInputStream& 
AIS::operator>>(char& c) {
  if (!(is >> c)) {
    throw IOException("AkentiInputStream:operator>>:(1)reading char failed");
  }

  return *this;
}

AkentiInputStream& 
AIS::operator>>(int& n) {
  if (!(is >> n)) {
    throw IOException("AkentiInputStream:operator>>:(2)reading int failed");
  }

  return *this;
}

AkentiInputStream& 
AIS::operator>>(bool& b) {
  if (!(is >> b)) {
    throw IOException("AkentiInputStream:operator>>:(3)reading bool failed");
  }

  return *this;
}

AkentiInputStream& 
AIS::operator>>(long& l) {
  if (!(is >> l)) {
    throw IOException("AkentiInputStream:operator>>:(4)reading long failed");
  }

  return *this;
}

AkentiInputStream& 
AIS::operator>>(double& d) {
  if (!(is >> d)) {
    throw IOException("AkentiInputStream:operator>>:(5)reading double failed");
  }

  return *this;
}

char 
AIS::get() {
  char c;

  if (!is.get(c)) {
    throw IOException("AkentiInputStream:get:reading char failed");
  }

  return c;
}

AkentiInputStream&
AIS::read(char* buffer, int len) {
  if (!(is.read(buffer, len))) {
    throw IOException("AkentiInputStream:read:reading failed");
  }
  
  return *this;
}

AkentiInputStream& 
AIS::operator>>(string& s) {
  StreamTokenizer st(is, " ");

  if (!st.hasMoreTokens()) {
    throw IOException("AkentiInputStream:operator>>:(6)input = estring");  
  }

  s = st.nextToken();

  while (st.hasMoreTokens() && s[s.length() - 1] == '\\') {
    s[s.length() - 1] = ' ';
    s += st.nextToken();
  } 

  return *this;
}

AkentiInputStream& 
AIS::operator>>(UtcTime& utcTime) {
  if (!(is >> utcTime)) {
    throw IOException("AkentiInputStream:read:reading a UtcTime failed");
  }

  return *this;
}

AkentiInputStream& 
AIS::operator>>(URL& url) {
  string temp;

  operator>>(temp);
  url = temp;
  return *this;
}

AkentiInputStream& 
AIS::operator>>(DistinguishedName& dn) {
  string temp;

  operator>>(temp);
  dn = temp;
  return *this;
}

AkentiInputStream& 
AIS::operator>>(AkentiObject& obj) {
  obj.readObject(*this);
  return *this;
}

} // end of namespace Akenti
