#include <iostream>
#include "ExpressionScanner.hpp"
#include "StringTokenizer.hpp"

using namespace Akenti;

typedef ExpressionScanner ES;

namespace Akenti {

static map<string, TokenType> TYPES = map<string, TokenType>();

static class TOKENSInitializer {

public:
  
  TOKENSInitializer() {
    TYPES["&&"] = AND;
    TYPES["||"] = OR;
    TYPES["!"]  = NOT;
    TYPES["="]  = EQ;
    TYPES["!="] = NEQ;
    TYPES[">"]  = GR;
    TYPES[">="] = GRE;
    TYPES["<"]  = LT;
    TYPES["<="] = LTE;
    TYPES["("]  = LPAREN;
    TYPES[")"]  = RPAREN;
    TYPES[""]   = INVALID;
  }
} tokenInitializer = TOKENSInitializer();

}; // end of namespace

ES::ExpressionScanner() : index(-1), lexeme(""), tokens() {
}

ES::~ExpressionScanner() {
}

void
ES::tokenize(const string& expression) {
  index = -1;
  lexeme = "";
  tokens.clear();

  StringTokenizer tokenizer(expression, "\n\t &!|=><()", true);
  vector<string> temp;

  while (tokenizer.hasMoreTokens()) {
    string token = tokenizer.nextToken();

    temp.push_back(token);
  }
  
  if (temp.size() == 0) {
    return;
  }

  int last = temp.size() - 1;

  for (int i = 0; i < last; ++i) {
    switch(temp[i][0]) {
      case '&':
      case '|':
        tokens.push_back(temp[i] + temp[i+1]);
	i++;
        break;
      case '!':
      case '<':
      case '>':
        if (temp[i + 1][0] == '=') {
	  tokens.push_back(temp[i] + temp[i+1]);
          i++;
        } else {
          tokens.push_back(temp[i]);
        }
        break;
      case ' ':
      case '\n':
      case '\t':
        // do nothing
        break;
      default:
        tokens.push_back(temp[i]);
    }
  }
  
  if (temp[last] != " ") {
    tokens.push_back(temp[last]);
  }
}

const string&
ES::getLexeme() const {
  return lexeme;
}

TokenType
ES::getTokenType() const {
  return TYPES[lexeme] ;
}

bool 
ES::hasMoreTokens() const {
  return index + 1 != tokens.size();
}

TokenType
ES::peek() const {
  int idx = index + 1;

  if (idx == tokens.size()) {
    return INVALID;
  }

  return TYPES[tokens[idx]];
}

void 
ES::nextToken() {
  if (!hasMoreTokens()) {
    lexeme = "";
    return;
  }
  
  lexeme = tokens[++index];

  if (getTokenType() != CONST) {
    return;
  }

  while (peek() == CONST) {
    lexeme += ' ';
    lexeme += tokens[++index];
  }
}

void
ES::listTokens() const {
  for (int i=0; i < tokens.size(); i++) {
    std::cout <<  i << tokens[i] << std::endl;
  }
}
