#include "ServerContext.hpp"
#include "CertificateFactory.hpp"
#include "KeyFactory.hpp"
 
using namespace Akenti;

typedef ServerContext SC;

ServerContext SC::context = ServerContext();

SC::ServerContext() {
}

SC::~ServerContext() {
}

const PrivateKey& 
SC::getPrivateKey() const {
  if (!privateKey) {
    throw Error("ServerContext:getPrivateKey:key is not set");
  }

  return privateKey;
}

const Certificate& 
SC::getCertificate() const {
  if (!certificate) {
    throw Error("ServerContext:getPrivateKey:key is not set");
  }

  return certificate;
} 

void
SC::unloadCredentials() {
  privateKey = NULL;
  certificate = NULL;
}

bool
SC::areCredentialsLoaded() const {
  return privateKey != NULL && certificate != NULL;
}

void 
SC::initPrivateKeyFromFile(const string& fileType, const string& fileName) {
  if (areCredentialsLoaded()) {
    throw Error("ServerContext:initPrivateKeyFromFile:cannot reset server's credentials");
  }

  KeyFactory kfac = KeyFactory::getInstance();
  
  privateKey = kfac->initPrivateKeyFromFile(fileType, fileName);
}

void 
SC::initCertificateFromFile(const string& fileType, const string& fileName) {
  if (areCredentialsLoaded()) {
    throw Error(
      "ServerContext:initCertificateFromFile:cannot reset server's credentials"
    );
  }

  CertificateFactory fac = CertificateFactory::getInstance();

  certificate = fac->initCertificateFromFile(fileType, fileName);
}

bool 
SC::isServerContextSet() {
  return context.areCredentialsLoaded();
}

ServerContext& 
SC::getServerContext() {
  return context; 
}
