#include <sstream>
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "AkentiRequest.hpp"
#include "Error.hpp"

typedef DOMImplementationRegistry DR;

using std::ostringstream;

namespace Akenti {

AkentiRequest::AkentiRequestImpl::AkentiRequestImpl(const string& tag) 
  : tag(tag), id(-1) {
  if (!tag.length()) {
    throw Error("AkentiRequestImpl::AkentiRequestImpl:illegal arg");
  }
}

AkentiRequest::AkentiRequestImpl::~AkentiRequestImpl() {
}

const string& 
AkentiRequest::AkentiRequestImpl::getTag() const {
  return tag;
}

void 
AkentiRequest::AkentiRequestImpl::setID(int id) {
  this->id = id;
}

int
AkentiRequest::AkentiRequestImpl::getID() const {
  return id;
}

void 
AkentiRequest::AkentiRequestImpl::toXML(DOMElement* request) const {
  DOMDocument *doc = request->getOwnerDocument();
  DOMElement *element = doc->createElement(XMLUtil::convert("ID"));

  char buffer[30];
  sprintf(buffer, "%d", id);

  element->appendChild(doc->createTextNode(XMLUtil::convert(buffer)));
  request->appendChild(element);
  doToXML(request);
}

void 
AkentiRequest::AkentiRequestImpl::fromXML(const DOMElement* request) {
  DOMNode *child = request->getFirstChild();
  DOMNode *temp  = child->getFirstChild();

  id = atoi(XMLUtil::convert(temp->getNodeValue()).c_str());
  doFromXML(request);
}

AkentiRequest::AkentiRequest() {
}

AkentiRequest::~AkentiRequest() {
}

AkentiRequest::AkentiRequest(AkentiRequestImpl* impl) : impl(impl) {
  if (!impl) {
    throw Error("AkentiRequest:AkentiRequest:illegal arg");
  }
}

const string& 
AkentiRequest::getTag() const {
  return impl->getTag();
}

void 
AkentiRequest::setID(int id) {
  impl->setID(id);
}

int
AkentiRequest::getID() const {
  return impl->getID();
}

string
AkentiRequest::toString() const {
  DOMImplementation* domImpl = DR::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = domImpl->createDocument(0, XMLUtil::convert(getTag().c_str()), 0);
  DOMElement* root = doc->getDocumentElement();
  ostringstream os;

  impl->toXML(root);
  XMLUtil::write(os, doc);
  delete(doc);
  return os.str();
}

void 
AkentiRequest::toXML(DOMElement* request) const {
  impl->toXML(request);
}

void 
AkentiRequest::fromXML(const DOMElement* request) {
  impl->fromXML(request);
}

}
