#include <sstream>
#include "Xerces-Includes.hpp"
#include "AkentiRequestResponseImplFactory.hpp"
#include "AkentiRequests.hpp"
#include "AkentiConstants.hpp"
#include "XMLUtil.hpp"
#include "Error.hpp"
#include "Exception.hpp"

using std::ostringstream;

typedef DOMImplementationRegistry DOM_REGISTRY;

namespace Akenti {

AkentiRequests::AkentiRequests() {
}

int 
AkentiRequests::numOfRequests() const {
  return requests.size();
}

void 
AkentiRequests::addRequest(const AkentiRequest& req) {
  requests.push_back(req);
}

const AkentiRequest& 
AkentiRequests::getRequestAt(int index) const {
  if (index < 0 || index >= requests.size()) {
    throw Error("AkentiRequests:getRequestAt:illegal arg");
  }

  return requests[index];
}

DOMElement*
AkentiRequests::toXML() const {
  int size = requests.size();

  if (!size) {
    throw Error("AkentiRequests:toXML:must have at least one request");
  }

  DOMImplementation* impl = DOM_REGISTRY::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument *doc = impl->createDocument(0, XMLUtil::convert("WORKAROUND"), 0);
  DOMElement *element = doc->createElement(XMLUtil::convert("akp:AkentiRequests"));
 
  element->setAttribute(XMLUtil::convert("xmlns:akp"),
                        XMLUtil::convert(CURRENT_PROT_NAME_SPACE));

  for (int i = 0; i < size; ++i) {
    string tag = "akp:" + requests[i].getTag();
    DOMElement* request = doc->createElement(XMLUtil::convert(tag.c_str()));

    requests[i].toXML(request);
    element->appendChild(request);
  }

  return element;
}

void 
AkentiRequests::fromXML(const DOMElement* element) {
  requests.clear();

  DOMNode* node = element->getFirstChild();

  do {
    string tag = XMLUtil::convert(node->getNodeName());
    int index = tag.find(":") + 1;

    tag = tag.substr(index, tag.length() - index);

    AkentiRequest req(AkentiRequestResponseImplFactory::getRequestImpl(tag));

    req.fromXML((DOMElement*) node);
    requests.push_back(req);
    node = node->getNextSibling();
  } while (node != NULL);
}

string 
AkentiRequests::toString() const {
  ostringstream os;
  DOMElement* element = toXML();

  XMLUtil::write(os, element);
  delete(element->getOwnerDocument());
  return os.str();
}

}
