#include <sstream>
#include "Xerces-Includes.hpp"
#include "AkentiRequestResponseImplFactory.hpp"
#include "AkentiResponses.hpp"
#include "XMLUtil.hpp"
#include "AkentiConstants.hpp"
#include "Error.hpp"
#include "Exception.hpp"

typedef DOMImplementationRegistry DOM_REGISTRY;

using std::ostringstream;

namespace Akenti {

AkentiResponses::AkentiResponses() {
}

int 
AkentiResponses::numOfResponses() const {
  return responses.size();
}

void 
AkentiResponses::addResponse(const AkentiResponse& req) {
  responses.push_back(req);
}

const AkentiResponse& 
AkentiResponses::getResponseAt(int index) const {
  if (index < 0 || index >= responses.size()) {
    throw Error("AkentiResponses:getResponseAt:illegal arg");
  }

  return responses[index];
}

DOMElement*
AkentiResponses::toXML() const {
  int size = responses.size();

  if (!size) {
    throw Error("AkentiResponses:toXML:must have at least one response");
  }

  DOMImplementation* impl = DOM_REGISTRY::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = impl->createDocument(0, XMLUtil::convert("WORKAROUND"), 0);
  DOMElement* element = doc->createElement(XMLUtil::convert("akp:AkentiResponses"));
 
  element->setAttribute(XMLUtil::convert("xmlns:akp"),
                        XMLUtil::convert(CURRENT_PROT_NAME_SPACE));

  for (int i = 0; i < size; ++i) {
    string tag = "akp:" + responses[i].getTag();
    DOMElement* response = doc->createElement(XMLUtil::convert(tag.c_str()));

    responses[i].toXML(response);
    element->appendChild(response);
  }
  
  return element;
}

void 
AkentiResponses::fromXML(const DOMElement* element) {
  responses.clear();

  DOMNode* node = element->getFirstChild();

  do {
    string tag = XMLUtil::convert(node->getNodeName());
    int index = tag.find(":") + 1;
 
    tag = tag.substr(index, tag.length() - index);

    AkentiResponse resp(AkentiRequestResponseImplFactory::getResponseImpl(tag));

    resp.fromXML((DOMElement*) node);
    responses.push_back(resp);
    node = node->getNextSibling();
  } while (node != NULL);
}

string
AkentiResponses::toString() const {
  ostringstream os;
  DOMElement* element = toXML();
 
  XMLUtil::write(os, element);
  delete(element->getOwnerDocument());
  return os.str();
}

}
