#ifndef AKENTI_CREATE_RESOURCE_REQUEST_HPP
#define AKENTI_CREATE_RESOURCE_REQUEST_HPP

#include "AkentiRequest.hpp"

namespace Akenti {

/**
  * Class to handle the arguments in a CreateResourceRequest.
  * This is the message that is sent to an Akenti server to
  * create a resource in the given location.
  * @see AkentiRequest
  */
class CreateResourceRequest : public AkentiRequest {
 
public:

  /**
    * Constructor
    */
  CreateResourceRequest();

  /**
    * Sets the name of the resource base to create.
    * Used by the client making the request.
    */
  void setResource(const string& resource);
 
  /**
    * Extracts the name of the resource from the message structure.
    * Used by the Akenti server when reading a CreateResource message.
    */
  const string& getResource() const;

private:

  friend class AkentiRequest;
  friend class AkentiRequestResponseImplFactory;
  static AkentiRequestImpl* getRequestImpl(); 
  class CreateResourceRequestImpl;
  typedef CreateResourceRequestImpl* ImplType; 

};

}

#endif
