#include "GetResourceInfoRequest.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"

namespace Akenti {

class GetResourceInfoRequest::GetResourceInfoRequestImpl : public AkentiRequestImpl {

public:

  GetResourceInfoRequestImpl() : AkentiRequestImpl("GetResourceInfoRequest") {
  }

  ~GetResourceInfoRequestImpl() {
  }

  void setResource(const string& resource) {
    this->resource = resource;
  }

  const string& getResource() const {
    return resource;
  }

  void doToXML(DOMElement* request) const {
    DOMDocument* doc = request->getOwnerDocument();
    DOMElement* certNode = doc->createElement(XMLUtil::convert("OpaqueCertificate"));
    DOMElement *resourceNode = doc->createElement(XMLUtil::convert("Resource"));

    resourceNode->appendChild(doc->createTextNode(XMLUtil::convert(resource.c_str())));
    request->appendChild(resourceNode);
  }
 
  void doFromXML(const DOMElement* request) {
    DOMNode* certNode = request->getFirstChild()->getNextSibling();

    resource = XMLUtil::convert(certNode->getFirstChild()->getNodeValue());
  }

private:

  string resource;

};

typedef GetResourceInfoRequest GRIR;

GRIR::GetResourceInfoRequest() 
  : AkentiRequest(getRequestImpl()) {
}

void 
GRIR::setResource(const string& resource) {
  GetResourceInfoRequestImpl* temp = (GetResourceInfoRequestImpl*) impl.get();

  temp->setResource(resource);
}

const string&
GRIR::getResource() const {
  GetResourceInfoRequestImpl* temp = (GetResourceInfoRequestImpl*) impl.get();

  return temp->getResource();
}

AkentiRequest::AkentiRequestImpl*
GRIR::getRequestImpl() {
  return new GetResourceInfoRequestImpl;
}

}
