#include "GetResourceInfoResponse.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Error.hpp"
#include "Exception.hpp"

namespace Akenti {

class GetResourceInfoResponse::GetResourceInfoResponseImpl 
  : public AkentiResponseImpl {

public:

  GetResourceInfoResponseImpl() : AkentiResponseImpl("GetResourceInfoResponse") {
  }

  ~GetResourceInfoResponseImpl() {
  }

  void GetResourceInfoResponseImpl::setResourceInfo(const string& resourceInfo) {
    this->resourceInfo = resourceInfo;
  }

  const string& getResourceInfo() const{
    return resourceInfo;
  }

  void doToXML(DOMElement* response) const{
    DOMDocument *doc = response->getOwnerDocument();
    DOMElement *child = doc->createElement(XMLUtil::convert("ResourceInfo"));

    child->appendChild(doc->createTextNode(XMLUtil::convert(resourceInfo.c_str())));
    response->appendChild(child);
  }

  void doFromXML(const DOMElement* response) {
    DOMNode *child = response->getLastChild();

    resourceInfo = "";

    if (child->getFirstChild()) {
      resourceInfo = XMLUtil::convert(child->getFirstChild()->getNodeValue());
    }
  }

private:

string resourceInfo;

};

GetResourceInfoResponse::GetResourceInfoResponse() 
  : AkentiResponse(getResponseImpl()) {
}

void 
GetResourceInfoResponse::setResourceInfo(const string& resourceInfo) {
  GetResourceInfoResponseImpl* temp = (GetResourceInfoResponseImpl *) impl.get();
  
  temp->setResourceInfo(resourceInfo);
}

const string&
GetResourceInfoResponse::getResourceInfo() const {
  GetResourceInfoResponseImpl* temp = (GetResourceInfoResponseImpl *) impl.get();

  return temp->getResourceInfo();
}

AkentiResponse::AkentiResponseImpl*
GetResourceInfoResponse::getResponseImpl() {
  return new GetResourceInfoResponseImpl;
}

}
