#include <sstream>
#include <algorithm>
#include <cctype>
#include "Error.hpp"
#include "Exception.hpp"
#include "OpaqueCertificate.hpp"
#include "AttributeCertificate.hpp"
#include "PolicyCertificate.hpp"
#include "UseCondCertificate.hpp"
#include "CapabilityCertificate.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Base64Util.hpp"
#include <iostream>

using namespace std;

namespace Akenti {
 
static string
constructHeader(bool begin, const string& type) {
  static const string B_HEADER = "-----BEGIN AKENTI ";
  static const string E_HEADER = "-----END AKENTI ";
  static const string C_HEADER = " CERTIFICATE-----";
  string ret = begin? B_HEADER : E_HEADER;
  string temp = type;
 
  // Shuts the compiler. 
  int (*op)(int) = std::toupper;
  std::transform(type.begin(), type.end(), temp.begin(), op);
  ret += temp;
  ret += C_HEADER;
  return ret;
}

typedef OpaqueCertificate OC;

void 
OC::toXML(const AkentiCertificate& cert, DOMElement* opaqueCert) {
  if (! cert) {
    throw Error("OpaqueCertificate:toXML:null AkentiCertificate");
  }

  opaqueCert->setAttribute(XMLUtil::convert("Type"), XMLUtil::convert(cert->getType().c_str()));

  DOMDocument* doc = opaqueCert->getOwnerDocument();
  DOMElement* temp = doc->createElement(XMLUtil::convert("EncodedCertificate"));

  ostringstream oss;

  AkentiCertificate::writeToStream("PEM", oss, cert);

  // TODO More efficiently
  // REMOVE PEM TAGS
  string pemCert = oss.str();
  int idx1 = pemCert.find('\n');
  int idx2 = pemCert.find("\n-", idx1);
 
  string rep = pemCert.substr(idx1, idx2 - idx1);

  rep = Base64Util::format(rep);

  temp->appendChild(doc->createTextNode(XMLUtil::convert(rep.c_str())));
  opaqueCert->appendChild(temp);
}

const AkentiCertificate
OC::fromXML(const DOMElement* opaqueCert) {
  string certType = XMLUtil::convert(opaqueCert->getAttribute(XMLUtil::convert("Type")));

  DOMNode* child = opaqueCert->getFirstChild();
  
  if (child == NULL || child->getNodeType() != DOMNode::ELEMENT_NODE) {
    throw Exception("OpaqueCertificate:fromXML:missing EncodedCertificate");
  }

  DOMNode* temp = child->getFirstChild();

  if (temp == NULL) {
    throw Exception("OpaqueCertificate:fromXML:missing EncodedCertificate");
  }

  // ADD PEM TAGS
  string pemCert = constructHeader(true, certType);
  string rep = XMLUtil::convert(temp->getNodeValue());
 
  rep = Base64Util::format(rep);

  if (rep[0] != '\n') {
    pemCert += '\n';
  }
 
  pemCert += rep;
 
  if (rep[rep.length() - 1] != '\n') {
    pemCert += '\n';
  }
 
  pemCert += constructHeader(false, certType);

  istringstream iss(pemCert);
  AkentiCertificate akCert = AkentiCertificate::initFromStream("PEM", iss);

  if (certType != akCert->getType()) {
    throw Exception("mis-match between certificate type listed in attribute and encoded certificate type");
  }

  return akCert;
}

}
