#include "Exception.hpp"
#include "PolicyInfo.hpp"
#include "CertificateInfo.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"

namespace Akenti {

typedef PolicyInfo PI;

PI::PolicyInfo() : resource("NO_RESOURCE"), policyCorrect(false) {
}

void
PI::setCertificateInfos(const vector<CertificateInfo>& certInfos) {
  this->certInfos = certInfos;
}

const vector<CertificateInfo>& 
PI::getCertificateInfos() const {
  return this->certInfos;
}

void
PI::setPolicyCorrect(bool policyCorrect) {
  this->policyCorrect = policyCorrect;
}

bool
PI::isPolicyCorrect() const {
  return this->policyCorrect;
}

void
PI::setResource(const string& resource) {
  if (!resource.length()) {
    throw Error("PolicyInfo:setResource:illegal arg");
  }

  this->resource = resource;
}

const string& 
PI::getResource() const {
  return this->resource;
}

void 
PI::toXML(DOMElement* policyInfo) const {
  policyInfo->setAttribute(XMLUtil::convert("PolicyCorrect"),
                           policyCorrect? XMLUtil::convert("true"):
                                          XMLUtil::convert("false"));

  policyInfo->setAttribute(XMLUtil::convert("Resource"),
                           XMLUtil::convert(resource.c_str()));


  DOMDocument* doc = policyInfo->getOwnerDocument();

  for (int i = 0; i < certInfos.size(); i++) {
    DOMElement* temp = doc->createElement(XMLUtil::convert("CertificateInfo"));

    certInfos[i].toXML(temp);
    policyInfo->appendChild(temp);
  }
}

void 
PI::fromXML(const DOMElement* policyInfo) {
  certInfos.clear();

  DOMNodeList* list = policyInfo->getElementsByTagName(XMLUtil::convert("CertificateInfo"));

  for (int i = 0; i < list->getLength(); i++) {
    CertificateInfo ci;
    DOMNode* child = list->item(i);
    
    ci.fromXML((DOMElement*) child);
    certInfos.push_back(ci);
  }

  resource = XMLUtil::convert(policyInfo->getAttribute(XMLUtil::convert("Resource")));
   
  const XMLCh* temp = policyInfo->getAttribute(XMLUtil::convert("PolicyCorrect"));

  policyCorrect = XMLUtil::convert(temp) == "true"? true : false;
}

}
