#include "HttpResponse.hpp"
#include "HttpConstants.hpp"
#include "StringTokenizer.hpp"
#include "IOException.hpp"
#include "InputStream.hpp"
#include "OutputStream.hpp"
#include <sstream>

using std::istringstream;

namespace Akenti {

HttpResponse::HttpResponse() : statusCode(-1) {
}

HttpResponse::~HttpResponse() {
}

void 
HttpResponse::setStatusCode(int statusCode) {
  this->statusCode = statusCode;
}

int 
HttpResponse::getStatusCode() const {
  return statusCode;
}

void 
HttpResponse::setStatusMessage(const string& statusMessage) {
  this->statusMessage = statusMessage;
}

string
HttpResponse::getStatusMessage() const {
  return statusMessage;
}

InputStream
HttpResponse::getInputStream() const {
  return HttpBase::getInputStream();
}


OutputStream
HttpResponse::getOutputStream() {
  return HttpBase::getOutputStream();
}


void 
HttpResponse::fromStream(InputStream& is) {
  string line;

  readLine(line, is);

  StringTokenizer tokenizer(line, " ");

  if (tokenizer.countTokens() < 3) {
    throw Exception("HttpRequestImpl:fromStream:failed reading header");
  }

  protocol = tokenizer.nextToken();

{
  // XXXX
  istringstream is(tokenizer.nextToken());
  is >> statusCode;
}

  // get rid of extra whitespace
  statusMessage = tokenizer.nextToken("").substr(1); 

  do {
    readLine(line, is);

    if (line.length() == 0) {
      break;
    }

    StringTokenizer tokenizer(line, ": ");

    if (tokenizer.countTokens() == 2) {
      string token = tokenizer.nextToken();

      if (token == "Content-type") {
        contentType = tokenizer.nextToken();
      } else if (token == "Content-length") {
        istringstream is(tokenizer.nextToken());

        is >> contentLength;
      }
    }

  } while (true);

  if (!contentLength) {
    return;
  }

  string body(contentLength, '\0');

  is->read((char*) body.c_str(), contentLength);

  if (!is->good()) {
    throw IOException("HttpRequestImpl:fromStream:failed reading body");
  }

  this->is = new istringstream(body);
}

void 
HttpResponse::toStream(OutputStream& os) const {
  os << protocol << ' ' << statusCode << ' ' << statusMessage << CRLF;

  if (contentType.length() != 0) {
    os << "Content-type" << ": " << contentType << CRLF;
  }

  int contentLength = 0;
  string body;

  if (obuff != NULL) {
    body = obuff->str();
    contentLength = body.length();
  }

  if (!contentLength) {
    os << CRLF;
    return;
  }

  os << "Content-length" << ": " << contentLength << CRLF;
  os << CRLF;
  os << body;
}

} // end of name space
