#include <sstream>
#include "Xerces-Includes.hpp"
#include "SoapEnvelope.hpp"
#include "InputStream.hpp"
#include "OutputStream.hpp"
#include "XMLUtil.hpp"
#include "AkentiConstants.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include <iostream>
#include <memory>

using std::istringstream;
using std::ostringstream;

using namespace std;

namespace Akenti {

SoapEnvelope::SoapEnvelope() : content(NULL) {
}

SoapEnvelope::SoapEnvelope(DOMElement* content) {
  if (content == NULL) {
    throw Error("SoapEnvelope:SoapEnvelope:illegal arg");
  }

  this->content = content;
}

SoapEnvelope::~SoapEnvelope() {
  if (content != NULL) {
    delete(content->getOwnerDocument());
  }
}

const DOMElement*
SoapEnvelope::getContent() const {
  return content;
}

DOMElement*
SoapEnvelope::releaseContent() {
  if (content == NULL) {
    throw Error("SoapEnvelope:releaseContent:illegal op");
  }

  DOMElement* ret = content;

  content = NULL;
  return ret;
}

void 
SoapEnvelope::fromStream(InputStream& is) {
  std::auto_ptr<AbstractDOMParser> parser = XMLUtil::parse(is);
  DOMDocument *doc = parser->getDocument();

  XMLUtil::filter(doc);

  DOMElement *root = doc->getDocumentElement();
  string temp = XMLUtil::convert(root->getNodeName());
  int index = temp.find(":") + 1;

  if (temp.substr(index, temp.length() - index) != "Envelope") {
    throw Exception("SoapEnvelope:fromStream:no envelope");
  }

  DOMNode *child  = root->getLastChild();

  if (child == NULL) {
    throw Exception("SoapEnvelope:fromStream:no soap body");
  } else if (child->getNodeType() != DOMNode::ELEMENT_NODE) {
    throw Exception("SoapEnvelope:fromStream:bad soap body");
  }

  DOMElement *body = (DOMElement *) child;

  temp = XMLUtil::convert(child->getNodeName());
  index = temp.find(":") + 1;

  if (temp.substr(index, temp.length() - index) != "Body") {
    throw Exception("SoapEnvelope:fromStream:no body");
  }

  child = body->getFirstChild();
  
  if (child == NULL) {
    throw Exception("SoapEnvelope:fromStream:no soap command");
  } else if (child->getNodeType() != DOMNode::ELEMENT_NODE) {
    throw Exception("SoapEnvelope:fromStream:no soap command");
  }

  // XXX CLONING:
  DOMDocument* newdoc = (DOMDocument*) doc->cloneNode(false);
  content = (DOMElement*) newdoc->importNode(child, true);
}

void 
SoapEnvelope::toStream(OutputStream& os) const{
  DOMDocument *doc = content->getOwnerDocument();
  DOMElement *root = doc->createElement(XMLUtil::convert("s:Envelope"));
  
  root->setAttribute(XMLUtil::convert("xmlns:s"), XMLUtil::convert(CURRENT_SOAP_NAME_SPACE));

  DOMElement *bodyNode = doc->createElement(XMLUtil::convert("s:Body"));

  root->appendChild(bodyNode);

  bodyNode->appendChild(content);
 
  // XXX
  ostringstream obuff;

  XMLUtil::write(obuff, root);

  os << obuff.str();
}

}
