#ifndef SOAP_ENVELOPE_HPP
#define SOAP_ENVELOPE_HPP

#include "Xerces-Decls.hpp"

namespace Akenti {

class InputStream;
class OutputStream;

  /**
    * Internal class to wrap a DOMElement containing XML content
    * These methods are called by SoapHttpResponseImpl and SoapHttpRequestImpl
    * and hide some of the details of soap messages.
    *
    */



class SoapEnvelope {

public:
  
  /**
   * Constructor - sets content to NUll
    */
  SoapEnvelope();

  /**
   * Destructor
    */
  ~SoapEnvelope();

  /**
   * Constructor - set content to the input value
    */
  SoapEnvelope(DOMElement* content);

  /**
    * Returns the content
    */
  const DOMElement* getContent() const;

  /**
    * Returns the content and then clears the stored value
    */
  DOMElement* releaseContent();

  /**
    * Initializes the content from an soap message read from the input stream
    */
  void fromStream(InputStream& is);

  /**
    * Writes the content as a soap message to the output stream
    */
  void toStream(OutputStream& os) const;

private:
  
  DOMElement *content;

};

}

#endif
