#include "Xerces-Includes.hpp"
#include "SoapHttpRequest.hpp"
#include "SoapEnvelope.hpp"
#include "XMLUtil.hpp"
#include "StringUtil.hpp"
#include "Error.hpp"
#include "Exception.hpp"

namespace Akenti {

SoapHttpRequest::SoapHttpRequest() : params(NULL) {
  setMethod("POST");
  setContentType("text/xml");
}

SoapHttpRequest::~SoapHttpRequest() {
  if (params != NULL) {
    delete(params->getOwnerDocument());
  }
}

void 
SoapHttpRequest::setParams(DOMElement* params) {
  // XXXX DOES IT HAVE A PARENT?
  if (params == NULL) {
    throw Error("SoapHttpRequest:Illegal arg");
  }

  this->params = params;
}

const DOMElement* 
SoapHttpRequest::getParams() const {
  if (params == NULL) {
    throw Error("SoapHttpRequest:getParams:Illegal arg");
  }

  return params;
}

InputStream 
SoapHttpRequest::getInputStream() const {
  throw Error("SoapHttpRequest:getInputStream:unsupported method");
}

OutputStream
SoapHttpRequest::getOutputStream() {
  throw Error("SoapHttpRequest:getOutputStream:unsupported method");
}

void 
SoapHttpRequest::fromStream(InputStream& in) {
  HttpRequest::fromStream(in);

  if (!StringUtil::equalsIgnoreCase("POST", getMethod())) {
    throw Exception("SoapHttpRequest:fromStream:bad soap method");
  } else if (!StringUtil::equalsIgnoreCase("text/xml", getContentType())) {
    throw Exception("SoapHttpRequest:fromStream:bad soap method");
  }

  SoapEnvelope env;

  in = HttpRequest::getInputStream();
  env.fromStream(in);
  params = env.releaseContent();
}

void 
SoapHttpRequest::toStream(OutputStream& os) const  {
  ostringstream* obuff = new ostringstream;
  OutputStream temp(obuff);

  SoapEnvelope env(params);

  env.toStream(temp);
  env.releaseContent();
  string body = obuff->str();

  os << getMethod() << " " << getURI() << " " << getProtocol() << CRLF;
  os << "Content-type" << ": " << getContentType() << CRLF;
  os << "Content-length" << ": " << body.length() << CRLF;
  os << CRLF;
  os << body;
}

} // end of name space
