#include <sstream>
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "SoapHttpResponse.hpp"
#include "StringUtil.hpp"
#include "SoapEnvelope.hpp"
#include "Exception.hpp"
#include "HttpConstants.hpp"
#include "Error.hpp"

using namespace std;

namespace Akenti {

SoapHttpResponse::SoapHttpResponse() : retValue(NULL) {
  faultCode = NONE;
  setContentType("text/xml");
}

SoapHttpResponse::~SoapHttpResponse() {
  if (retValue != NULL) {
    delete(retValue->getOwnerDocument());
  }
}

const DOMElement*
SoapHttpResponse::getReturnValue() const {
  return retValue;
}

void 
SoapHttpResponse::setStatusCode(int statusCode) {
  switch(statusCode) {
    case 200:
      faultCode = NONE;
      HttpResponse::setStatusCode(HttpConstants::HTTP_OK);
      break;
    case 400:
      faultCode = CLIENT;
      HttpResponse::setStatusCode(HttpConstants::HTTP_BAD);
      break;
    case 500:
      faultCode = SERVER;
      HttpResponse::setStatusCode(HttpConstants::HTTP_ERROR);
      break;
    default:
      throw Error("SoapHttpResponse:setStatusCode:unknown code");
  }
}

void 
SoapHttpResponse::setReturnValue(DOMElement* retValue) {
  this->retValue = retValue;
}

void 
SoapHttpResponse::setFaultCode(SoapFaultCode faultCode) {
  this->faultCode = faultCode;
}

SoapFaultCode
SoapHttpResponse::getFaultCode() const {
  return faultCode;
}

void
SoapHttpResponse::setFaultMessage(const string& faultMessage) {
  this->faultMessage = faultMessage;
}

const string&
SoapHttpResponse::getFaultMessage() const {
  return faultMessage;
}

InputStream
SoapHttpResponse::getInputStream() const {
  throw Error("SoapHttpResponse:getInputStream:unssuported");
}

OutputStream 
SoapHttpResponse::getOutputStream() {
  throw Error("SoapHttpResponse:getOutputStreamStream:unssuported");
}

void
SoapHttpResponse::fromStream(InputStream& is) {
  HttpResponse::fromStream(is);

  if (!StringUtil::equalsIgnoreCase("text/xml", getContentType())) {
    throw Exception("SoapHttpRequestImpl:fromStream:bad content");
  }

  InputStream tempStream = HttpResponse::getInputStream();

  if (!tempStream) {
    throw Exception("SoapHttpRequestImpl:fromStream:no soap body");
  }

  SoapEnvelope env;
  
  env.fromStream(tempStream);
   
  DOMElement *element = env.releaseContent();
  DOMNode *node = element->getFirstChild();

  retValue = (DOMElement*) node; 
}

void
SoapHttpResponse::toStream(OutputStream& os) const {
  ostringstream* obuff = new ostringstream;
  OutputStream temp(obuff);

  if (faultCode != NONE) {
    DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(XMLUtil::convert("Core"));
    DOMDocument *doc = impl->createDocument(0, XMLUtil::convert("WORKAROUND"), 0);
    DOMElement *fault = doc->createElement(XMLUtil::convert("s:Fault"));
    DOMElement *code = doc->createElement(XMLUtil::convert("faultcode"));

    code->appendChild(doc->createTextNode(XMLUtil::convert(faultCode == SERVER? "Server" : "Client")));
    fault->appendChild(code);

    DOMElement *message = doc->createElement(XMLUtil::convert("faultstring"));

    message->appendChild(doc->createTextNode(XMLUtil::convert(getStatusMessage().c_str())));
    fault->appendChild(message);

    SoapEnvelope env(fault);

    env.toStream(temp);
  } else {
    DOMDocument *doc = retValue->getOwnerDocument();
    DOMElement *element = doc->createElement(XMLUtil::convert("ReturnValue"));

    element->appendChild(retValue);

    SoapEnvelope env(element);

    env.toStream(temp);
    env.releaseContent();
  }

  string body = obuff->str();

  os << getProtocol() << ' ' << getStatusCode() << ' ' << getStatusMessage() << CRLF;
  os << "Content-type" << ": " << contentType << CRLF;
  os << "Content-length" << ": " << body.length() << CRLF;
  os << CRLF;
  os << body;
}

} // end of name space
