#ifndef AK_SOCKET_IMPL_FACTORY_HPP
#define AK_SOCKET_IMPL_FACTORY_HPP

namespace Akenti {

class SocketImpl;

/**
  * This abstract class provides applications with socket generation.
  *
  * @version 1.1 00/11/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class SocketImplFactory {

public:

  /**
    * Creates the object.
    */
  SocketImplFactory();

  /**
    * Destroys the object.
    */
  virtual 
  ~SocketImplFactory();
  
  /**
    * Creates a new SocketImpl object.
    *
    * @return the new SocketImpl object
    */
  virtual SocketImpl* 
  createSocketImpl() const;

  /**
    * Sets this object to the new SocketImplFactory fac.
    * 
    * @param fac the new SocketImplFactory
    */ 
  static void 
  setSocketImplFactory(const SocketImplFactory& fac);

  /**
    * Returns the SocketImplFactory object.
    *
    * @return the SocketImplFactory object 
    */ 
  static const SocketImplFactory& 
  getSocketImplFactory();

private:

  static SocketImplFactory fac;

};

} // end of namespace Akenti

#endif
