#ifndef AK_URL_HANDLER_HPP
#define AK_URL_HANDLER_HPP

#include "ref_ptr.hpp"
#include "URLHandlerImpl.hpp"

/**
  * A <i>URLHandler</i> object is a <b>reference-count</b> pointer that wraps
  * a <i>URLHandlerImpl</i>. Provides services indirectly from URLHandlerImpl.
  *
  * @see URLHandlerImpl
  * @see ref_ptr 
  *
  * @version 1.1 00/11/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

namespace Akenti {

class URLHandler : public ref_ptr<URLHandlerImpl> {

public:

  /**
    * Create this object. 
    *
    * @param impl the URL handler
    */   
  URLHandler(URLHandlerImpl* impl = NULL) 
    : ref_ptr<URLHandlerImpl> (impl) {
  }
  
  /** 
    * Decrements the ref_cnt by 1. When ref_cnt drops to 0, destroys the
    * URLHandlerImpl and this object.
    */ 
  ~URLHandler() {
  }

};

} // end of namespace Akenti

#endif
