#include <sstream>
#include "URLHandlerImpl.hpp"
#include "URL.hpp"
#include "StringTokenizer.hpp"
#include "Error.hpp"
#include "MalformedURLException.hpp"

using std::istringstream;
using std::ostringstream;

typedef Akenti::URLHandlerImpl UHI;

UHI::URLHandlerImpl() {
}

UHI::~URLHandlerImpl() {
}

void 
UHI::parseURL(URL& url, const string& spec) const {
  StringTokenizer tokenizer(spec, "/:", true);

  if (!tokenizer.hasMoreTokens()) {
    return;
  }

  string token = tokenizer.nextToken();
 
  if (token != "/") {
    if (tokenizer.hasMoreTokens()) { 
      setFile(url, token + tokenizer.nextToken(""));
    } else {
      setFile(url, token);
    }

    return;
  } else if (!tokenizer.hasMoreTokens()) {
    return;
  }

  token = tokenizer.nextToken();

  if (token != "/") {
    if (tokenizer.hasMoreTokens()) { 
      setFile(url, token + tokenizer.nextToken(""));
    } else {
      setFile(url, token);
    }
   
    return;
  } else if (!tokenizer.hasMoreTokens()) {
    return;
  }

  token = tokenizer.nextToken();
  
  if (token == "/") {
    if (tokenizer.hasMoreTokens()) { 
      setFile(url, tokenizer.nextToken(""));
    }
    
    return;
  }

  setHost(url, token);

  if (tokenizer.hasMoreTokens()) {
    token = tokenizer.nextToken();

    if (token == ":") {
      if (!tokenizer.hasMoreTokens()) {
        return;
      }

      istringstream is(tokenizer.nextToken("/"));

      int port;

      if ((is >> port) && port > 0) {
        setPort(url, port);
      }
    } 

    if (tokenizer.hasMoreTokens()) {
      setFile(url, tokenizer.nextToken(""));
    }
  }
}

void
UHI::initURL(URL& url, const string& host, int port, const string& file) const {
  setHost(url, host);
  setPort(url, port);
  setFile(url, file);
}

string
UHI::toExternalForm(const URL& url) const {
  ostringstream os;

  os << url.getProtocol() << ':';

  string host = url.getHost();

  if (host.length()) {
    os << "//" << host; 
  }
  
  int port = url.getPort(); 

  if (port != -1) {
    os << ":" << port;
  }
 
  os << url.getFile();

  return os.str();
}

void 
UHI::setHost(URL& url, const string& host) const {
  url.setHost(host);
}

void 
UHI::setPort(URL &url, int port) const {
  url.setPort(port);
}

void 
UHI::setFile(URL& url, const string& file) const {
  if (file.length()) {
    if (file[0] == '/') {
      url.setFile(file);
    } else {
      url.setFile('/' + file);
    }
  }
}
