#include <string>
#include <openssl/x509.h>
#include "OpenSSLBase64Impl.hpp"
#include "SecurityException.hpp"

typedef Akenti::OpenSSLBase64Impl OSBI;
typedef unsigned char * UCPtr;

OSBI::OpenSSLBase64Impl() : Base64Impl(), buffer() {
}
 
OSBI::~OpenSSLBase64Impl() {
}

void 
OSBI::doEncodeInit() {
  buffer.erase();
  EVP_EncodeInit(&ctx);
}

void 
OSBI::doEncodeUpdate(const string& data, int offset, int len){
  string temp(2 * (ctx.num + len), '\0');   // we must take ctx.num into account :)
  int out;

  EVP_EncodeUpdate(
    &ctx, 
    (UCPtr) temp.c_str(),
    &out, 
    (UCPtr) data.c_str() + offset,
    len
  );
  buffer.append(temp, 0, out);
}

string 
OSBI::doEncode() {
  string temp(2 * ctx.num + 5, '\0');
  int out;

  EVP_EncodeFinal(
    &ctx, 
    (UCPtr) temp.c_str(), 
    &out
  );
 
  buffer.append(temp, 0, out);
  return buffer.substr(0, buffer.length() - 1); /* here we remove the last newline */
}

void 
OSBI::doDecodeInit() {
  buffer.erase();
  EVP_DecodeInit(&ctx);
}

void 
OSBI::doDecodeUpdate(const string& enc, int offset, int len){
  string temp(ctx.num + len, '\0');   // we must take ctx.num into account :)
  int out;

  int ret = EVP_DecodeUpdate(
    &ctx, 
    (UCPtr) temp.c_str(), 
    &out, 
    (UCPtr) enc.c_str() + offset,
    len
  );

  if (ret  == -1) {
     throw SecurityException("OSBI:doDecodeUpdate:decoding failed");
  }

  buffer.append(temp, 0, out);
}

string 
OSBI::doDecode(){
  string temp(ctx.num, '\0');
  int out;

  int ret = EVP_DecodeFinal(
    &ctx, 
    (UCPtr) temp.c_str(),
    &out
  );

  if (ret == -1){
    throw SecurityException("OSBI:doDecode:decoding failed");
  }

  buffer.append(temp, 0, out);
  return buffer; 
}


