#include "OpenSSLCipherImpl.hpp"
#include "SecurityException.hpp"

typedef Akenti::OpenSSLCipherImpl OSCI;
typedef unsigned char * UCPtr;

OSCI::OpenSSLCipherImpl(const string& algo) : CipherImpl(algo) {
  if (!EVP_get_cipherbyname(algo.c_str())) {
    throw Error("OSCI::OpenSSLCipherImpl:alg not supported");
  }
}

OSCI::~OpenSSLCipherImpl() {
}

void 
OSCI::resetEngine() {
  EVP_CIPHER_CTX_init(&ctx);

  const EVP_CIPHER* cipher = EVP_get_cipherbyname(alg.c_str());

  int status = (state == ENCRYPTION) ? 
    EVP_EncryptInit(&ctx, cipher, (UCPtr) key.getBytes(), NULL) :
    EVP_DecryptInit(&ctx, cipher, (UCPtr) key.getBytes(), NULL)
  ;

  if (!status) {
    throw SecurityException("OSCI::resetEngine:can't initialize context");
  }
}

string 
OSCI::doUpdate(const string& data, int offset, int len) {
  const EVP_CIPHER* cipher = EVP_get_cipherbyname(alg.c_str());
  int size = EVP_CIPHER_block_size(cipher);
  string ret((len / size) * size, '\0');
  int status = (state == ENCRYPTION) ?
    EVP_EncryptUpdate(&ctx,(UCPtr)ret.data(),&len,(UCPtr)data.data()+offset, len):
    EVP_DecryptUpdate(&ctx,(UCPtr)ret.data(),&len,(UCPtr)data.data()+offset, len)
  ;

  if (!status) {
    throw SecurityException("OSCI::doUpdate:EVP_Update failed");
  }
  
  ret.resize(len);
  return ret;
}

string 
OSCI::doDoFinal() {
  const EVP_CIPHER* cipher = EVP_get_cipherbyname(alg.c_str());
  string ret(EVP_CIPHER_block_size(cipher), '\0');
  int len;

  int status = (state == ENCRYPTION) ?
    EVP_EncryptFinal(&ctx, (UCPtr) ret.data(), &len):
    EVP_DecryptFinal(&ctx, (UCPtr) ret.data(), &len)
  ;

  if (!status) {
    throw SecurityException("OSCI::doDoFinal:EVP_Final failed");
  }
  
  ret.resize(len);
  return ret;
}
