#ifndef OPEN_SSL_CONTEXT_IMPL_HPP
#define OPEN_SSL_CONTEXT_IMPL_HPP

#include <openssl/ssl.h>
#include "SSLContextImpl.hpp"

namespace Akenti {

/**
  * A wrapper around SSL_CTX whic is used to store the credentials of
  * a server/client and the trusted ca certs.
  *
  * @author Shrilekha Mudumbai 
  * @author Abdelilah Essiari
  */

class OpenSSLContextImpl : public SSLContextImpl {

public:

  /**
    * Internal class.
    *
    * @param Version version the version of openSSL that is supported.
    *        currently it must be TLS1 or SSL V3.
    * @param bool isClient true for client context false otherwise 
    */
  OpenSSLContextImpl(Version version, bool isClient);
  
  /**
   * Destructor
   */
  ~OpenSSLContextImpl();

  /**
    * Sets the identity certificate chain c of the client/server.
    */
  void setCertificateFile(const string& certFile, FileType type);

  /**
    * Sets the private key of the client/server.
    */
  void setPrivateKeyFile(const string& keyFile, FileType type);

  /**
    */
  void setTrustedCAS(const string& filename);

  /**
    */
  void addTrustedCA(const Certificate& cert);

protected:

  /**
    */
  void doSetVerify(int verify);

private:
  
  friend class OpenSSLSecureSocketImpl;

  SSL_CTX* ctx;

};

} // end namespace Akenti

#endif
