#include <openssl/evp.h>
#include "OpenSSLMessageDigestImpl.hpp"
#include "SecurityException.hpp"

typedef Akenti::OpenSSLMessageDigestImpl OSMI;

OSMI::OpenSSLMessageDigestImpl(const string &algo) : MessageDigestImpl(algo) {
  const EVP_MD *md = EVP_get_digestbyname((char *) algo.c_str());

  if (!md) {
    throw SecurityException(
      "OpenSSLMessageDigestImpl:Constructor:No Such Algorithm" + algo
    ); 
  }

  EVP_DigestInit(&context, md);
}

OSMI::~OpenSSLMessageDigestImpl() {
}

void 
OSMI::doUpdate(const string& data, int offset, int len) {
  unsigned char * ptr = (unsigned char *) data.c_str();

  EVP_DigestUpdate(
    &context, 
    ptr + offset, 
    len
  );
}

string 
OSMI::doDigest() {
  string digest(EVP_MD_CTX_size(&context), '\0');

  EVP_DigestFinal(&context, (unsigned char *) digest.data(), NULL);
  return digest;
}

void 
OSMI::reset() {
  EVP_DigestInit(&context, EVP_MD_CTX_md(&context));
}
