#ifndef OPENSSL_PROVIDER_IMPL_HPP
#define OPENSSL_PROVIDER_IMPL_HPP 

#include <string>
#include "ProviderImpl.hpp"
#include "KeyFactory.hpp"
#include "CertificateFactory.hpp"
#include "SSLContext.hpp"

using std::string;

namespace Akenti {

/**
  * This class uses the Open SSL crypto library.
  *
  * @see ProviderImpl
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class OpenSSLProviderImpl : public ProviderImpl {

public:

  /**
    */
  OpenSSLProviderImpl();

  /**
    */
  ~OpenSSLProviderImpl();

  /**
    */
  void loadDigestAlgorithms();

  /**
    */
  bool isDigestAlgorithmSupported(const string& alg) const;

  /**
    */
  MessageDigest getMessageDigest(const string& alg) const;

  /**
    */
  void loadSignatureAlgorithms();

  /**
    */
  bool isSignatureAlgorithmSupported(const string& alg) const;

  /**
    */
  Signature getSignature(const string& alg) const;

  /**
    */
  Hmac getHmac(const string& alg) const;

  /**
    */
  void loadCipherAlgorithms();

  /**
    */
  bool isCipherAlgorithmSupported(const string& alg) const;
  
  /**
    */
  Cipher getCipher(const string& alg) const ;

  /**
    */
  KeyStore getKeyStore(const string& type) const;

  /**
    */
  bool seedRandomGenerator(const string& seed);

  /**
    * Returns a <i>Base64</i> encoder.
    */
  Base64 getBase64() const;

  /**
    */
  KeyFactory getKeyFactory() const;

  /**
    */
  CertificateFactory getCertificateFactory() const;

  /**
    */
  SecureSocketImpl* createSecureSocketImpl(const SSLContext& ctx) const;

  /**
    */
  SSLContext createSSLContext(SSLConstants::Version version, bool isClient) const;

  /**
    */
  string toDerString(const DistinguishedName& dn) const;

};

} // end namespace Akenti

#endif
