#include <openssl/x509.h>
#include "OpenSSLUtil.hpp"
#include "DistinguishedName.hpp"
#include "Certificate.hpp"
#include "OpenSSLX509CertificateImpl.hpp"
#include "Error.hpp"

typedef Akenti::OpenSSLUtil OSU;
typedef unsigned char * UCPtr;

string
OSU::toDer(const DistinguishedName& dn) {
  X509_NAME* name = X509_NAME_new();

  if (name == NULL) {
    throw Error("OpenSSLUtil:toDer:bad alloc");
  }
  
  populateX509Name(name, dn);
  int len = i2d_X509_NAME(name, NULL);

  if (len <= 0) {
    X509_NAME_free(name);
    throw Error("OpenSSLUtil:toDer:should never happen");
  }

  string ret(len, '\0');
  unsigned char *temp = (unsigned char *) ret.data();

  i2d_X509_NAME(name, &temp);
  X509_NAME_free(name);
  return ret;
}

void 
OSU::populateX509Name(X509_NAME *name, const DistinguishedName& dn) {
  for (int i = 0; dn.hasMorePairs(i); i++) {
    const pair<string, string>& pair = dn.getPairAt(i);
    string attr = pair.first;
    int type = V_ASN1_PRINTABLESTRING;
 
    if (attr == "DC") {
      type = V_ASN1_IA5STRING;
    } else if (attr == "emailAddress") {
      type = V_ASN1_IA5STRING;
    } else if (attr == "Email") {
      attr = "emailAddress";
      type = V_ASN1_IA5STRING;
    }

    X509_NAME_add_entry_by_txt(
      name, 
      (char *) attr.c_str(),
      type,
      (UCPtr) pair.second.c_str(),
      pair.second.length(),
      i,
      1
    );
  }
}

Akenti::Certificate
OSU::fromC_X509(X509* cert) {
  if (!cert) {
    throw Error("SSLUtil:fromC_X509:illegal argument");
  }

  CRYPTO_add(&cert->references, 1, CRYPTO_LOCK_X509);
  return new OpenSSLX509CertificateImpl(cert);
}
