#ifndef OPEN_SSL_UTIL_HPP
#define OPEN_SSL_UTIL_HPP

#include <string>
#include <openssl/x509.h>

using std::string;

namespace Akenti {

class DistinguishedName;
class Certificate;

/**
  * Provides some functions that can de/endcode strings and functions to 
  * populate X509 name with distinguished name or with the principal.
  *
  * @see DistinguishedName
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class OpenSSLUtil {

public:

  /**
    */
  static string toDer(const DistinguishedName& dn);

  /**
    * Populates the X509 name with the given distinguished name.
    *  
    * @param name the X509 name
    * @param dn the distinguished name used for populating the X509 name
    */
  static void populateX509Name(X509_NAME *name, const DistinguishedName& dn);

  /**
    * Converts X509 to the C++ Certificate class.
    */
  static Certificate fromC_X509(X509* cert);
};

} // end namespace Akenti

#endif
