#ifndef AK_CERTIFICATE_REQUEST_GENERATOR_HPP
#define AK_CERTIFICATE_REQUEST_GENERATOR_HPP

#include "ref_ptr.hpp"
#include "CertificateRequestGeneratorImpl.hpp"

/**
  * A <i>CertificateRequestGenerator<i> object is a 
  * <i>reference-count</i> pointer that wraps a 
  * <i>CertificateRequestGeneratorImpl</i> object.
  * It provides applications, albeit indirectly, the methods
  * of a CertificateRequestGenerator which include setting subject
  * publicKey, versionNumber, generating a CertificateRequest with
  * specified algorithm and privateKey, displaying the Certificate
  * in a string.
  *
  * @see ref_ptr
  * @see CertificateRequestGeneratorImpl
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

namespace Akenti {

class CertificateRequestGenerator : public ref_ptr<CertificateRequestGeneratorImpl> {

public:

  /**
    * Constructs a <i>CertificateRequestGenerator</i> given a 
    * <i>CertificateRequestGeneratorImpl</i> object.
    *
    * @param impl  wrapped <i>CertificateRequestGeneratorImpl</i> object
    * @see ref_ptr
    * @see CertificateRequestGeneratorImpl
    */
  CertificateRequestGenerator(CertificateRequestGeneratorImpl* impl = NULL) 
    : ref_ptr<CertificateRequestGeneratorImpl> (impl) {  
    
  }
  
  /**
    * Decrements the count of references to the wrapped 
    * <i>CertificateRequestGeneratorImpl</i> object.
    * If this count is zero, then the wrapped object is deleted.
    *
    * @see ref_ptr
    */
  ~CertificateRequestGenerator() {
  }
  
};

} // end of namespace Akenti

#endif
