#ifndef AK_CIPHER_HPP
#define AK_CIPHER_HPP 

#include "ref_ptr.hpp"
#include "CipherImpl.hpp"
#include "Provider.hpp"

namespace Akenti {

/**
  *
  * DES-EDE, DES-ECB, DES-CBC
  * 
  * @see Key
  * @author Abdelilah Essiari
  */
class Cipher : public ref_ptr<CipherImpl> {

public:

  /**
    */
  Cipher(CipherImpl* impl = NULL) : ref_ptr<CipherImpl>(impl) {
  }

  /**
    * Destroys this cipher.
    */
  ~Cipher() {
  }

  /**
    * Returns <i>true</i> if <i>alg</i> is supported.
    *
    * Note: Assumes loadCiphers has been called.
    *
    * @see loadCiphers()
    */
  static bool isAlgSupported(const string& alg) {
    Provider& po = Provider::getProvider();

    return po->isCipherAlgorithmSupported(alg);
  }

  /**
    */
  static Cipher getInstance(const string& alg) {
    Provider& po = Provider::getProvider();

    return po->getCipher(alg);
  }

  /**
    */
  static Cipher getNullCipher() {
    return new CipherImpl("NULL");
  }

};

}
#endif
