#ifndef AK_PUBLIC_KEY_HPP
#define AK_PUBLIC_KEY_HPP 

#include "PublicKeyImpl.hpp"
#include "ref_ptr.hpp"
#include "KeyFactory.hpp"

namespace Akenti {

/**
  * A <i>PublicKey</i> object is a <b>reference-count</b> pointer
  * that wraps a <i>PublicKeyImpl</i> object.
  *
  * @see ref_ptr
  * @see PublicKeyImpl
  * @see KeyPair#getPublicKey()
  * @see KeyFactoryImpl#getKeyEncoded(const string&, const PublicKey&)
  * @see KeyFactoryImpl#initPublicKey(const string&, const string&,
  *                                   const string&)
  * @see KeyFactoryImpl#generateRSAKeyPair(int, unsigned long)
  * @see KeyFactoryImpl#generateDSAKeyPair(int, const string&)
  * @see SignatureImpl#initVerify(const PublicKey&)
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class PublicKey : public ref_ptr<PublicKeyImpl> {

public:

  /**
    * Constructs a <i>PublicKey</i> given a <i>PublickeyImpl</i>
    * object. Typically a <i>KeyFactory</i> is used to create an instance of
    * this class.
    *
    * @param impl wrapped <i>PublicKeyImpl</i> object.
    * @see PublicKeyImpl
    * @see ref_ptr
    * @see KeyFactoryImpl#initPublicKey(const string&, const string&, 
    *                                   const string&)
    * @see KeyFactoryImpl#generateRSAKeyPair(int, unsigned long)
    * @see KeyFactoryImpl#generateDSAKeyPair(int, const string&)
    * @see KeyPair#getPublicKey()
    */
  PublicKey(PublicKeyImpl *impl = NULL)
     : ref_ptr<PublicKeyImpl> (impl) {
  }

  /**
    * Decrements the count of references to the wrapped <i>PublickeyImpl</i>
    * object. If this count is zero, then the wrapped object is deleted.
    *
    * @see ref_ptr
    */
  ~PublicKey() {
  }

};

} // end of namespace Akenti

#endif
