#ifndef AK_REVOKED_CERTIFICATE_HPP
#define AK_REVOKED_CERTIFICATE_HPP

#include "ref_ptr.hpp"
#include "RevokedCertificateImpl.hpp"

namespace Akenti {

/**
  * A <i>RevokedCertificate<i> object is a <i>reference-count</i> pointer
  * that wraps a <i>RevokedCertificateImpl</i> object. It provides
  * information, albeit indirectly of the certificate has been revoked. 
  *
  * @see ref_ptr
  * @see RevokedCertificateImpl
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class RevokedCertificate : public ref_ptr<RevokedCertificateImpl> {
public:

  /**
    * Constructs a <i>RevokedCertificate</i> given a 
    * <i>RevokedCertificateImpl</i> object. 
    *
    * @param impl wrapped <i>RevokedCertificateImpl</i> object.
    * @see RevokedCertificateImpl
    * @see ref_ptr
    */
  RevokedCertificate(RevokedCertificateImpl *impl = NULL) :
  	ref_ptr<RevokedCertificateImpl> (impl) {
  }

  /**
    * Decrements the count of references to the wrapped 
    * <i>RevokedCertificateImpl</i> object. If this count is zero, 
    * then the wrapped object is deleted.
    *
    * @see ref_ptr
    */
  ~RevokedCertificate() {
  }

};

} // end of namespace Akenti

#endif
