#ifndef AK_SECRET_KEY_HPP
#define AK_SECRET_KEY_HPP

#include <string>

using std::string;

namespace Akenti {

/**
  * This class is used for symmetric keys.
  *
  * @author Abdelilah Essiari
  */
class SecretKey {

public:

   /**
     * Constructs a symmetric key.
     *
     * @param alg string name of key's algorithm
     * @param key string key's binary representation
     */
   SecretKey(const string& alg = "NO_ALG", const string& key ="") 
      : alg(alg), key(key) {
   }

   /**
     */
   ~SecretKey() {
   }

   /**
     * Returns the name of the algorithm. 
     */
   const string& getAlgorithm() const {
     return alg;
   }

   /**
     * Returns the size of this <i>SecretKey</i>.
     */
   int getSize() const {
     return key.size();
   }

   /**
     * Returns the bytes that make up this <i>SecretKey</i>.
     */
   const string& toString() {
     return key; 
   }

   /**
     * Returns the bytes that make up this <i>SecretKey</i>.
     */
   const unsigned char* getBytes() const {
     return (unsigned char*) key.data();
   }

private:

   string key;

   string alg;

};

}

#endif
