#include <iostream>
#include <sstream>
#include "DistinguishedName.hpp"
#include "Exception.hpp"
#include "Util.hpp"
#include "StringTokenizer.hpp"

using namespace std;

namespace Akenti {

bool 
Util::compare(const DistinguishedName& dn1, const DistinguishedName& dn2) {
  // XXX
  // Exceptions???
  DistinguishedName temp1(getHashDN(dn1));
  DistinguishedName temp2(getHashDN(dn2));
  int index = 0;
  bool ret = true;

  while (ret && temp1.hasMorePairs(index)) {
    const pair<string, string>& pair = temp1.getPairAt(index++);
    
    ret = temp2.containsAttributeValue(pair.first, pair.second);
  }
  
  return ret;
}

static const char* USED_ATTRIBUTES[] = {
  "CN", "C", "O", "OU", "DC",  
  "cn", "c", "o", "ou", "dc", 
  NULL 
};

string 
Util::getHashDN(const DistinguishedName& dn) {
  int index = 0;
  string ret;

  while (dn.hasMorePairs(index)) {
    const pair<string, string>& pair = dn.getPairAt(index++);
    bool found = false;

    for (int i = 0; !found && USED_ATTRIBUTES[i] != NULL; ++i) {
      found = (pair.first == USED_ATTRIBUTES[i]);
    }

    if (found) {
      ret += '/';
      ret += pair.first; 
      ret += '='; 
      ret += pair.second; 
    }
  }
  
  return ret;
} 

string 
Util::createFileName(const string& digest) {
  long hash = (long) digest[0] & 0x000000ffL;

  hash = hash | (((long) digest[1]<<8L) & 0x0000ffffL);
  hash = hash | (((long) digest[2]<<16L) & 0x00ffffffL);
  hash = hash | (((long) digest[3]<<24L) & 0xffffffffL);

  std::ostringstream os;

  os << std::hex << hash;

  string rep = os.str();
  int len = rep.size();

  if (len == 8) {
    return rep;
  }

  return string(8 - len, '0') + rep;
}

#include <time.h>
#include <netdb.h>
#include <unistd.h>

string
Util::createUniqueID(const string& id) {
  char buff[100];
  string ret;

  memset(buff, 0, 100);

  if (!gethostname(buff, 100)) {
    struct hostent* hp = gethostbyname(buff);

    if (hp) {
      ret += hp->h_name;
      ret += '#';
    }

    memset(buff, 0, 100);
  }

  ret += id;
  ret += '#';
  
  time_t dtime = time(NULL);
  struct tm *stime = gmtime(&dtime);
  strftime(buff, 100, "%c", stime);

  ret += buff;
  return ret;
}

string
Util::fText(int indentAmt, const string& str) {
  const int LIMIT = 70;

  string indentStr = indent(indentAmt) + str;

  // string is less than the LIMIT no need to wrap

  if (indentStr.length() <= LIMIT) {
    return indentStr;
  }

  // start wrapping everything under the ':'

  string::size_type idx = indentStr.find(':');

  int subIndent;

  if (idx != string::npos) {
    subIndent = idx + 2;
  } else {
    subIndent = indentAmt;
  }

  // start the wrapping

  string formatText;
  string leftOver = indentStr;
  int length = leftOver.length();
  string temp;


  while (length > LIMIT) {
    idx = leftOver.find('\n');

    // gets rid of new lines in string
    if (idx != string::npos) {
      temp = leftOver;
      leftOver = temp.substr(0, idx);
      leftOver += temp.substr(idx + 1);
    }

    formatText += leftOver.substr(0, LIMIT) + '\n';
    leftOver = indent(subIndent) + leftOver.substr(LIMIT);
    length = leftOver.length();
  }

  if (length != 0) {
    formatText += leftOver;
  }

  formatText += '\n';

  return formatText;
}

string
Util::indent(int indentAmt) {
  string indentString; 

  for (int i = 0; i < indentAmt; i++) {
    indentString += ' ';     
  }
  return indentString; 
} 

string
Util::normalizedString(string oldStr) {
   string newStr;
   stringstream in;
   ostringstream out;
   string tmp;

   in << oldStr;
   while(in >> ws >> tmp)
       out << tmp;

   newStr = out.str();
   //cout << "SRS: old = " << oldStr << ", new = " << newStr << endl;
   return newStr;
}


}
