
#include <stdio.h>
#include "AkClientInf.h"
#include "cap.h"
#include "eval.h"
#include <time.h>
#include <string.h>

/* call back function to evaluate a single attribute and value pair 
 * Called by CondAct_evaluate
 */

int doEvaluate(ExprToken op, const AttrInfo info) {
 const char* attribute = Attr_getAttributeName(info);
 const char* value = Attr_getValue(info);
 int match;
 time_t curTime;
 struct tm *lTime;
 int hValue;
 int mValue;
 int i;
 char *c = value;
 char tempC[10];

 if (strcmp(attribute,"TIME") != 0) {  /* only know about TIME */
    return 0;
 }
 /* assume value is a 24 hr based hr:min string*/
 if (strlen(c) > 5) { 
    return 0;
 }
 c = strchr(c,':');
 if (c != NULL) {
    strncpy(tempC,value, c-value);
    hValue = atoi(tempC);
    strcpy(tempC,c+1);
    mValue = atoi(tempC);
 } else { 
    hValue = atoi(c);
    mValue = 0;
 }
  
 curTime = time(&curTime);
 lTime = localtime(&curTime);
 
 switch (op){
	 case AK_LT: if ((lTime->tm_hour < hValue) || 
		      ((lTime->tm_hour == hValue) && (lTime->tm_min < mValue)))
	    return 1;
	    else return 0;
	 case AK_LTE: if ((lTime->tm_hour < hValue) ||
		      ((lTime->tm_hour == hValue) && (lTime->tm_min <= mValue)))
	    return 1;
	    else return 0;
         case AK_GR: if ((lTime->tm_hour > hValue) ||
		      ((lTime->tm_hour == hValue ) && (lTime->tm_min > mValue)))
	    return 1;
	    else return 0;
         case AK_GRE: if (( lTime->tm_hour > hValue) ||
		       ((lTime->tm_hour == hValue) && (lTime->tm_min >= mValue)))
	    return 1;
	    else return 0;
         case AK_EQ: if ((lTime->tm_hour == hValue) && (lTime->tm_min == mValue))
	    return 1;
	    else return 0;
	 default: return 0;
 }
 return 0;
}

int main (int argc, char *argv[]) {

  char *status = NULL;
  FILE *cert;
  char certS[5000];
  char *rs;
  AKMessage repStatus = NULL;
  CapCert cap = NULL;
  CondActions cAct;
  int i,j;

  if (argc < 4 ) {
    printf ("Usage is ClientInfTest <UserCertificate> <Resource> <config-file> \n");
    exit (-1);
  }

  akentiClientInit(argv[3],&status);
  printf ("%s\n",status);

  cert = fopen(argv[1],"r");
  if (cert == NULL){
     printf ("failed to open %s\n",argv[1]);
     exit (-1);
  }

  rs = (char *) malloc (1024);
  rs = fgets(rs, 1024, cert);
  certS[0] = '\0';
  while (rs != NULL) {
     strcat(certS,rs);
     rs = fgets(rs, 1024, cert);
  }
 
  printf ("Checking the access of %s to %s\n",argv[1], argv[2]);

  if ( remoteCheckAccessByCertString(certS, argv[2], &cap, &repStatus) == 0 ) {
     printf ("Status: %d  %s\n",AKMsg_getCode(repStatus),AKMsg_getDescription(repStatus));
     printf ("Actions granted are: ");
     for (i=0; i < CapCert_numOfGrantedActions(cap); i++){
	printf ("%s ", CapCert_grantedActionAt(cap,i));
     }
     printf ("\n\n");
     for (i=0;i <  CapCert_numOfCondActions(cap); i++){
	printf ("Conditional actions are: \n");
	cAct = CapCert_condActionsAt(cap,i);
	if (CondAct_isCritical(cAct)) {
	   printf ("Critical constraint is %s \n", CondAct_getConstraint(cAct));
	} else { 
	   printf ("Non-critical constraint is %s ", CondAct_getConstraint(cAct));
	}
	printf ("Possible actions granted are: ");
	for (j = 0; j < CondAct_numOfActions(cAct); j++) {
	   printf ("%s ", CondAct_actionAt(cAct,j));
	}
	printf ("\n");
	if (CondAct_evaluate(cAct,(EvalCallBack)doEvaluate, NULL)) {
	   printf ("actions are granted\n");
	} else {
	   printf ("actions are NOT granted\n");
	}
     }
  } else { 
    printf(" Call failed: %d %s\n",AKMsg_getCode(repStatus),AKMsg_getDescription(repStatus));
  }
  CapCert_free(cap);
  AKMsg_free(repStatus);
  exit (0);
}
