#!/bin/sh

# A test of running the C++ AkentiServer and the C++ remoteClient

# Hopefully, this test will work. TS

orig_dir=`pwd`

###
# First, start the C++ server
###

# Make sure someone else isn't using port 8443
echo "Checking if port 8443 is free..."
netstat -at | grep 8443 | grep -v TIME_WAIT > /dev/null
ns_ret=$?
if [ "${ns_ret}" = "0" ]; then
  echo "  Port 8443 looks busy.  Sorry, you loose."
  exit 1
fi

# Get server config file and turn off Caching and SSLMode
config_file=../../test/resourceTree/conf/akentiConf
if [ -f ${config_file} ]; then
  echo "Writing config file into ./akentiConf"
  rm -f ./akentiConf
  cp ${config_file} akentiConf
else
  echo "Can't find config file: ${config_file}"
  exit 1
fi

# Get client config file and turn off Caching and SSLMode
client_config_file=../../test/resourceTree/conf/clientConf
if [ -f ${client_config_file} ]; then
  echo "Writing config file into ./clientConf"
  rm -f ./clientConf
  cp ${client_config_file} clientConf
else
  echo "Can't find config file: ${client_config_file}"
  exit 1
fi

# Check for akenti
if [ ! -x ../../tools/akenti ]; then
  echo "cannot find ../../tools/akenti"
  exit 1
fi

# Start the server
echo "Starting AkentiServer..."
../../servers/akentiServer/AkentiServer -f ./akentiConf > AkentiServer.log 2>&1 &
svr_pid=$!
echo "output in ./AkentiServer.log"

sleep 1

# Here are a few functions for exiting properly
clean_up()
{
  echo "Killing server (pid=${svr_pid})"
  kill $svr_pid
  wait $svr_pid 2> /dev/null
}

failed()
{
  clean_up
  echo "Test FAILED.  Here's the AkentiServer output:"
  cat AkentiServer.log
  exit $1
}

###
# Now, start the remoteClient
###

echo "Requesting CheckAccess..."
../../tools/akenti RCA -F ./clientConf -r R2 -q -x ../resourceTree/idCerts/userICert.fa924567.0
ret_val=$?
if [ ! "${ret_val}" = "0" ]; then failed ${ret_val}; fi

echo "Requesting CheckPolicy..."
../../tools/akenti RCP -F ./clientConf -r R1
ret_val=$?
if [ ! "${ret_val}" = "0" ]; then failed ${ret_val}; fi

echo "Requesting ResourceBrowse..."
../../tools/akenti RRB -F ./clientConf -r R1
ret_val=$?
if [ ! "${ret_val}" = "0" ]; then failed ${ret_val}; fi

echo "Requesting Create Resource..."
../../tools/akenti RCR -F ./clientConf -r R2/S1/T2
ret_val=$?
if [ ! "${ret_val}" = "1" ]; then failed ${ret_val}; fi

echo "Requesting Upload Certificate..."
../../tools/akenti RUP -F ./clientConf -p ../resourceTree/R2/S1/htauthority 
ret_val=$?
if [ ! "${ret_val}" = "1" ]; then failed ${ret_val}; fi

echo "Requesting Delete Certificate..."
../../tools/akenti RDP -F ./clientConf -r R2/S1
ret_val=$?
if [ ! "${ret_val}" = "1" ]; then failed ${ret_val}; fi

clean_up
echo "Test succeeded"
