#include <sstream>
#include <iostream>
#include <fstream>

#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"

#include "../shared/TestCaseConstants.hpp"
#include "AttributeCertGeneratorTestCase.hpp"
#include "AttributeCertGenerator.hpp"
#include "DistinguishedName.hpp"
#include "PrivateKey.hpp"
#include "KeyFactory.hpp"
#include "Util.hpp"
#include "SecurityConstants.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "XMLUtil.hpp"

using namespace std;
using namespace Akenti;

typedef AttributeCertGeneratorTestCase ACGTC;

void 
ACGTC::generateAttributeCertificates() {
  string method;

  try {
    method = "AttributeCertGeneratorTestCase:generateAttributeCert1:";
    generateAttributeCert1();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage()); 

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE()); 
  } catch(Error& err) {
    CppUnit::Message m(method +  err.getMessage()); 

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE()); 
  }
}

void
ACGTC::generateAttributeCert1() {
#if DEBUG
  cout << "\n Generating USERI's Attribute Certificate 1 ..." << endl;
#endif

  UtcTime now;
  UtcTime after(365 * 24 * 60 * 60);

  DistinguishedName ca(CAIA_DN);
  DistinguishedName issuer(SHI_DN);
  AkentiPrincipal guarantor(issuer, ca);

  DistinguishedName user(USERI_DN);
  AkentiPrincipal principal(user, ca);

  AttributeCertGenerator gen;

  gen.setVersionNumber(2);
  gen.setUniqueID("I");
  gen.setNotBefore(now);
  gen.setNotAfter(after);
  gen.setIssuer(guarantor);
  gen.setAttribute("JOB");
  gen.setValue("SUPERVISOR");
  gen.setPrincipal(principal);

  KeyFactory kfac = KeyFactory::getInstance();
  PrivateKey privKey = kfac->initPrivateKeyFromFile("PEM", ID_DIR + SHI_KEY);

  gen.sign(privKey, SecurityConstants::RSA_MD5);

  AkentiCertificate cert = gen.generate();
  string fileName = AKCERT_DIR + user.getValue("CN");

  fileName += ".ATTR." + Util::createFileName(cert->hashCode()) + ".0";
 
  ofstream os(fileName.c_str());

  AkentiCertificate::writeToStream(AK_CERT_ENC_TYPE, os, cert);
  os.close();
  CPPUNIT_ASSERT(!os.fail());

{
   ostringstream os;
 
   AkentiCertificate::writeToStream("PEM", os, cert);
   istringstream is(os.str());
   AkentiCertificate certFromStream = AkentiCertificate::initFromStream("PEM", is);

   CPPUNIT_ASSERT(cert == certFromStream);
}

{
   ostringstream os;
 
   AkentiCertificate::writeToStream("XML", os, cert);
   istringstream is(os.str());
   AkentiCertificate certFromStream = AkentiCertificate::initFromStream("XML", is);

   CPPUNIT_ASSERT(cert == certFromStream);
}
 
 
#if DEBUG
  cout << "\nSaved USERI's  Attribute Certificate" << fileName << endl;
#endif
}

CppUnit::TestSuite*
ACGTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("AttributeCertGeneratorTestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<AttributeCertGeneratorTestCase> (
      "generateAttributeCertificates",
      &AttributeCertGeneratorTestCase::generateAttributeCert1
    )
  );

  return testSuite;
}
