#include "cppunit/TextTestRunner.h"

#include <iostream>
#include "Provider.hpp"
#include "OpenSSLProviderImpl.hpp"
#include "PolicyCertGeneratorTestCase.hpp"
#include "UseCondCertGeneratorTestCase.hpp"
#include "AttributeCertGeneratorTestCase.hpp"
#include "CapabilityCertGeneratorTestCase.hpp"
#include "XMLUtil.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace Akenti; 
using namespace std;

int
main(int ac, char **argv) {
  try {
    Provider po = Provider(new OpenSSLProviderImpl);
    
    po->loadDigestAlgorithms();
    po->loadSignatureAlgorithms();

    Provider::setProvider(po);
    XMLUtil::init();

    CppUnit::TextTestRunner runner;

    runner.addTest(PolicyCertGeneratorTestCase::suite());
    runner.addTest(UseCondCertGeneratorTestCase::suite());
    runner.addTest(AttributeCertGeneratorTestCase::suite());
    runner.addTest(CapabilityCertGeneratorTestCase::suite());
    runner.run("", false);
    return 0;
  } catch (Exception& ex) {
    cout << "TestCerts:main:" + ex.getMessage() << endl;
  } catch (Error& err) {
    cout << "TestCerts:main:FATAL" + err.getMessage() << endl;
  }

  return 1;
}
