#include <sstream>
#include <vector>

#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"

#include "AkentiOutputStream.hpp"
#include "AkentiInputStream.hpp"
#include "ElementsTestCase.hpp" 
#include "DistinguishedName.hpp"
#include "AkentiPrincipal.hpp"
#include "AttributeInfo.hpp"
#include "AkentiIssuer.hpp"
#include "Condition.hpp"
#include "ConditionalActions.hpp"
#include "UseCondIssuerGroup.hpp" 
#include "Exception.hpp"
#include "Error.hpp"

typedef ElementsTestCase ETC;

using namespace Akenti;
using namespace std;

void
ETC::elementsReadWrite() {
  try {

    DistinguishedName dn("/C=US/O=LBNL/OU=DSD/CN=John Brown");
    DistinguishedName ca("/C=US/O=LBNL/OU=DSD/CN=Jane Smith");
    AkentiPrincipal ap(dn, ca);
    vector<DistinguishedName> cas(1, ca);
    vector<AkentiPrincipal> aps(1, ap);

    // Akenti attribute
    AttributeInfo attrInfo1 (true, "group", "DSD", aps);

    // X509 attribute
    AttributeInfo attrInfo2 ("CN", "User I", cas);

    Directory idDir("http://home.netscape.com/");
    vector<Directory> idDirs(1, idDir);

    // AkentiIsser

    AkentiIssuer akentiIssuer(ap, idDirs);    

    // Condition 

    vector<AttributeInfo> attrInfos;
    attrInfos.push_back(attrInfo1);
    attrInfos.push_back(attrInfo2);

    Condition cond("OU = LBL && JOB = SUPERVISOR", attrInfos);

    // ConditionalActions 

    string action1 = "READ";
    string action2 = "WRITE";

    vector<string> actions;
    actions.push_back(action1);
    actions.push_back(action2);

    ConditionalActions condActions(cond, actions, false);

    // UseCondIssuerGroup 

    Directory ucDir("file:/usr/local/UC");
    vector<Directory> ucDirs(1, ucDir);

    UseCondIssuerGroup useCondIssuerGroup(aps, ucDirs);

    ostringstream os;

    AkentiOutputStream aos(os);

    aos << akentiIssuer
        << ' ' 
        << condActions; 

    istringstream is(os.str());
    AkentiInputStream ais(is);

    AkentiIssuer tempAkentiIssuer;
    ais >> tempAkentiIssuer;

    ConditionalActions tempCondActions;
    ais >> tempCondActions;
  } catch(Exception& ex) {
    CppUnit::Message m("elementReadWrite:" + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error &err) {
    CppUnit::Message m("elementReadWrite:FATAL:" + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

CppUnit::TestSuite*
ETC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("ElementsTestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<ElementsTestCase>(
      "elementsReadWrite",
      &ElementsTestCase::elementsReadWrite
    )
  );

  return testSuite;
} 
