#include <string>
#include <iostream>
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"

#include "../shared/TestCaseConstants.hpp"
#include "AkentiEngineTestCase.hpp"
#include "AkentiInit.hpp"
#include "AkentiPrincipal.hpp"
#include "AkentiEngine.hpp"
#include "PolicyContext.hpp"
#include "CapabilityCertificate.hpp"
#include "ConditionalActions.hpp"
#include "AkentiMessage.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace std;
using namespace Akenti;

typedef AkentiEngineTestCase AETC;

#define DEBUG 0

void 
AETC::doTest() {
  string method;

  try {
    method = "AkentiEngineTestCase:runROOT1Test:";
    runROOT1Test();
    method = "AkentiEngineTestCase:runROOT2Test:";
    runROOT2Test();
    method = "AkentiEngineTestCase:runSUB1Test:";
    runSUB1Test();
    method = "AkentiEngineTestCase:runSUB3Test:";
    runSUB3Test();
    method = "AkentiEngineTestCase:runSUB4Test:";
    runSUB4Test();
    method = "AkentiEngineTestCase:runROOT3Test:";
    runROOT3Test();
    method = "AkentiEngineTestCase:runROOT4Test:";
    runROOT4Test();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
AETC::runROOT1Test() {
#if DEBUG
  cout << "\nBEGIN runROOT1TesT" << endl;
#endif

  AkentiPrincipal user(USERI_DN, CAIA_DN);
  CapabilityCertificate cert;
  AkentiEngine eng; 
  AkentiMessage msg;

  bool ret = eng.checkAccess(ROOT1, user, cert, msg);

#if DEBUG 
  cout << "Akenti Message:" << msg.toString() << endl;
  
  if (ret) {
    cout << "Expected Actions for this user are: READ" << endl;
    printActions(cert->getUnConditionalActions()); 
  }

  cout << "END runROOT1Test" << endl;
#endif
  
  CPPUNIT_ASSERT(ret);
}

void
AETC::runROOT2Test() {
#if DEBUG
  cout << "\nBEGIN runROOT2Test" << endl;
#endif

  AkentiPrincipal user(USERI_DN, CAIA_DN);
  AkentiEngine eng;
  PolicyContext ctx;
  AkentiMessage msg;

  CPPUNIT_ASSERT(eng.checkAccess(ROOT2, user, ctx, msg));


#if DEBUG
  cout << "Akenti Message: " << msg.toString() << endl;
  cout << "Expected Actions for this user are: READ" << endl;
  CapabilityCertificate cert = ctx->getCapabilityCert();

  printActions(cert->getUnConditionalActions());
  cout << "END runROOT2Test" << endl;
#endif
}

void
AETC::runSUB1Test() {
#if DEBUG
  cout << "\nBEGIN runSUB1Test" << endl;
#endif

  AkentiPrincipal user(USERI_DN, CAIA_DN);
  AkentiEngine eng;
  CapabilityCertificate cert;
  AkentiMessage msg;

  CPPUNIT_ASSERT(eng.checkAccess(SUB1, user, cert, msg));

#if DEBUG
  cout << "Akenti Message: " << msg.toString() << endl;
  cout << "Expected Actions for this user are: READ, WRITE, EXECUTE" << endl;
  printActions(cert->getUnConditionalActions());

  cout << "END runSUB1Test" << endl;
#endif
}

void
AETC::runSUB3Test() {
#if DEBUG
  cout << "\nBEGIN runSUB3Test" << endl;
#endif

  AkentiPrincipal user(USERI_DN, CAIA_DN);
  AkentiEngine eng;
  CapabilityCertificate cert;
  AkentiMessage msg;

  CPPUNIT_ASSERT(eng.checkAccess(SUB3, user, cert, msg));

#if DEBUG
  cout << "Akenti Message: " << msg.toString() << endl;
  cout << "Expected Actions for this user are: READ WRITE" << endl;
  printActions(cert->getUnConditionalActions());
  cout << "END runSUB3Test" << endl;
#endif
}

void
AETC::runSUB4Test() {
#if DEBUG
  cout << "\nBEGIN runSUB4Test" << endl;
#endif

  string config = CONF_DIR + CONF_FILE;
  string status;
  bool init = AkentiInit::init(config, status);

#if DEBUG
  cout << status << endl;
#endif

  CPPUNIT_ASSERT(init);
  AkentiPrincipal user(USERI_DN, CAIA_DN);

  AkentiEngine eng;
  PolicyContext ctx;
  AkentiMessage msg;

  bool ret = eng.checkAccess(SUB4, user, ctx, msg);

#if DEBUG
  cout << "Akenti Message: " << msg.toString() << endl;
  CPPUNIT_ASSERT(ret);
  cout << "Expected Actions for this user are: READ, WRITE, EXECUTE" << endl;
  CapabilityCertificate cert = ctx->getCapabilityCert();
  printActions(cert->getUnConditionalActions());

  cout << "END runrunSUB4Test" << endl;
#endif
}

void
AETC::runROOT3Test() {
#if DEBUG
  cout << "\nBEGIN runROOT3TesT" << endl;
#endif

  AkentiPrincipal user(USERI_DN, CAIA_DN);
  CapabilityCertificate cert;
  AkentiEngine eng;
  AkentiMessage msg;

  bool ret = eng.checkAccess(ROOT3, user, cert, msg);

#if DEBUG
  cout << "Akenti Message:" << msg.toString() << endl;

  if (ret) {
    cout << "Expected Actions for this user are: READ" << endl;
    printActions(cert->getUnConditionalActions());
    printActions(cert->getConditionalActions());
  }

  cout << "END runROOT3Test" << endl;

#endif

  CPPUNIT_ASSERT(ret);
}

void
AETC::runROOT4Test() {
#if DEBUG
  cout << "\nBEGIN runROOT4TesT" << endl;
#endif

  AkentiPrincipal user(USERI_DN, CAIA_DN);
  CapabilityCertificate cert;
  AkentiEngine eng;
  AkentiMessage msg;

  bool ret = eng.checkAccess(ROOT4, user, cert, msg);

#if DEBUG
  cout << "Akenti Message:" << msg.toString() << endl;

  if (ret) {
    cout << "Expected Actions for this user are: NONE" << endl;
    printActions(cert->getUnConditionalActions());
    printActions(cert->getConditionalActions());
  }

  cout << "END runROOT4Test" << endl;

#endif

  CPPUNIT_ASSERT(ret);
}

void 
AETC::printActions(const vector<string>& actions) {
  cout << "The actions granted are: ";

  int size = actions.size();

  if (size == 0) {
    cout << "NONE" << endl;
    return;
  }

  cout << actions[0];

  for (int i = 1; i < actions.size(); ++i) {
    cout << ", " << actions[i];
  }

  cout << endl;
}

void 
AETC::printActions(const vector<ConditionalActions>& actions) {
  cout << "Conditional actions are: ";

  int size = actions.size();

  if (size == 0) {
    cout << "NONE" << endl;
    return;
  }

  cout << actions[0].toLogString();

  for (int i = 1; i < actions.size(); ++i) {
    cout << ", " << actions[i].toLogString();
  }

  cout << endl;
}


CppUnit::TestSuite*
AETC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("AkentiEngineTestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<AkentiEngineTestCase>( 
      "doTest", 
      &AkentiEngineTestCase::doTest
    )
  );

  return testSuite;
}

