#include <iostream>
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "CheckAccessRequestTestCase.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "Xerces-Includes.hpp"
#include "AkentiRequests.hpp"
#include "CheckAccessRequest.hpp"
#include "AkentiPrincipal.hpp"

using namespace Akenti;
using namespace std;

typedef CheckAccessRequestTestCase CARTC;

void 
CARTC::doTest() {
  string method;

  try {
    method = "CheckAccessRequestTestCase:doTest:testRequestWithPrincipal:";
    testRequestWithPrincipal();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
CARTC::testRequestWithPrincipal() {
#if DEBUG
  cout << "Entering testRequestWithPrincipal ..." << endl;
#endif

  AkentiRequests reqs;
  int id = 1;
  string resource("SOME_RESOURCE");

  CheckAccessRequest car1;

  car1.setID(id);
  car1.setResource(resource);

  DistinguishedName ca("/CN=JOE");
  DistinguishedName user("/CN=JOE_CA");
  AkentiPrincipal principal(user, ca);

  car1.setAkentiPrincipal(principal);
  reqs.addRequest(car1);

#if DEBUG
  cout << reqs.toString() << endl;
#endif 

  DOMElement* element = reqs.toXML();

  reqs.fromXML(element);
  delete(element->getOwnerDocument());

  CPPUNIT_ASSERT(reqs.numOfRequests() == 1);

  CheckAccessRequest car2 = (CheckAccessRequest&) reqs.getRequestAt(0);

  CPPUNIT_ASSERT(principal == car1.getAkentiPrincipal());
  CPPUNIT_ASSERT(id == car1.getID());
  CPPUNIT_ASSERT(resource == car1.getResource());
  CPPUNIT_ASSERT(! car1.containsX509Chain());
  CPPUNIT_ASSERT(id == car2.getID());
  CPPUNIT_ASSERT(resource == car2.getResource());
  CPPUNIT_ASSERT(principal == car2.getAkentiPrincipal());

#if DEBUG
  cout << "Exiting testRequestWithPrincipal ..." << endl;
#endif
}

CppUnit::TestSuite*
CARTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("CheckAccessRequestTestCase");
 
  testSuite->addTest (
    new CppUnit::TestCaller<CheckAccessRequestTestCase> (
      "doTest",
      &CheckAccessRequestTestCase::doTest
    )
  );
 
  return testSuite;
}

