#include <iostream>
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "CheckPolicyRequestTestCase.hpp"
#include "CheckPolicyRequest.hpp"
#include "AkentiRequests.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace Akenti;
using namespace std;

typedef CollectPolicyRequestTestCase CPRTC;


void 
CPRTC::doTest() {
  string method;

  try {
    method = "CollectPolicyRequestTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
CPRTC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif

  string resource("Gammasphere");
  int id = 1;
  CheckPolicyRequest cpr1;

  cpr1.setResource(resource);
  cpr1.setID(id);

  CPPUNIT_ASSERT(resource == cpr1.getResource());
  CPPUNIT_ASSERT(id == cpr1.getID());

  AkentiRequests reqs;
  
  reqs.addRequest(cpr1);

  DOMElement* element = reqs.toXML();

  reqs.fromXML(element);
#if DEBUG
  XMLUtil::write(cout, element);
#endif
  delete(element->getOwnerDocument());

  CheckPolicyRequest cpr2 = (CheckPolicyRequest&) reqs.getRequestAt(0);

  CPPUNIT_ASSERT(resource == cpr2.getResource());
  CPPUNIT_ASSERT(id == cpr2.getID());

#if DEBUG
  cout << reqs.toString() << endl;
  cout << "\nExiting test1 ..." << endl;
#endif
}

CppUnit::TestSuite*
CPRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("CollectPolicyRequestTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<CollectPolicyRequestTestCase> ( 
      "doTest", 
      &CollectPolicyRequestTestCase::doTest
    )
  );

  return testSuite;
}
