#include <iostream>
#include "cppunit/TestCaller.h"
#include "cppunit/Exception.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "Xerces-Includes.hpp"
#include "DistinguishedName.hpp"
#include "PrivateKey.hpp"
#include "PublicKey.hpp"
#include "KeyFactory.hpp"
#include "SecurityConstants.hpp"
#include "../shared/TestCaseConstants.hpp"
#include "CertificateFactory.hpp"
#include "AttributeCertGenerator.hpp"
#include "CheckPolicyResponseTestCase.hpp"
#include "CheckPolicyResponse.hpp"
#include "AkentiResponses.hpp"
#include "XMLUtil.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace std;

typedef CheckPolicyResponseTestCase CPRTC;

void 
CPRTC::doTest() {
  string method;

  try {
    method = "CheckPolicyResponseTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
CPRTC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif

  AkentiCertificate cert = genAttributeCert();
  string msg("CERTIFICATE IS FINE");
  AkentiMessage mesg(104, msg);
  vector<AkentiMessage> v(1, mesg);

  CertificateInfo ci;

  ci.setCertificate(cert);
  ci.setCertificateMessages(v);

  vector<CertificateInfo> cis(1, ci);
  PolicyInfo pi;
  
  pi.setCertificateInfos(cis);

  vector<PolicyInfo> pis;

  pis.push_back(pi);
  pis.push_back(pi);

  CheckPolicyResponse cpr;

  cpr.setPolicyInfos(pis);
  cpr.setAkentiMessage(AkentiMessage(109, "Check Policy OK"));

  AkentiResponses resps;

  resps.addResponse(cpr);

  DOMElement* element = resps.toXML();

  resps.fromXML(element);
#if DEBUG 
  XMLUtil::write(cout, element);
#endif

  delete(element->getOwnerDocument());
  CPPUNIT_ASSERT(1 == resps.numOfResponses());


  cpr = (CheckPolicyResponse&) resps.getResponseAt(0);
  // ...

#if DEBUG
  cout << resps.toString() << endl;

  cout << "\nExiting test1 ..." << endl;
#endif
}

CppUnit::TestSuite*
CPRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("CheckPolicyResponseTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<CheckPolicyResponseTestCase> ( 
      "doTest", 
      &CheckPolicyResponseTestCase::doTest
    )
  );

  return testSuite;
}

AkentiCertificate 
CPRTC::genAttributeCert() {
#if DEBUG
  cout << "\n Generating Attribute Certificate ..." << endl;
#endif

  UtcTime now;
  UtcTime after(365 * 24 * 60 * 60);

  DistinguishedName ca(CAIA_DN);
  DistinguishedName issuer(SHI_DN);
  AkentiPrincipal guarantor(issuer, ca);

  DistinguishedName user(USERI_DN);
  AkentiPrincipal principal(user, ca);

  AttributeCertGenerator gen;

  gen.setVersionNumber(2);
  gen.setUniqueID("I");
  gen.setNotBefore(now);
  gen.setNotAfter(after);
  gen.setIssuer(guarantor);
  gen.setAttribute("JOB");
  gen.setValue("SUPERVISOR");
  gen.setPrincipal(principal);

  KeyFactory kfac = KeyFactory::getInstance();
  PrivateKey privKey = kfac->initPrivateKeyFromFile("PEM", ID_DIR + SHI_KEY);

  gen.sign(privKey, SecurityConstants::RSA_MD5);

  AkentiCertificate cert = gen.generate();

  return cert;
}

