#include <fstream>
#include <iostream>
#include <sstream>
#include "Exception.hpp"
#include "Error.hpp"
#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "CreateResourceRequestTestCase.hpp"
#include "CreateResourceRequest.hpp"
#include "AkentiRequests.hpp"

using namespace std;

typedef CreateResourceRequestTestCase CRRTC;

void 
CRRTC::doTest() {
  string method;

  try {
    method = "CreateResourceRequestTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

/**
 * Tests toXML and fromXML
 */
void
CRRTC::test1() {
#if DEBUG
  cout << "Entering test1! ..." << endl;
#endif
 
  AkentiRequests reqs;
  CreateResourceRequest req;
  string resource = "R1";

  req.setResource(resource);
  CPPUNIT_ASSERT(resource == req.getResource());
 
  reqs.addRequest(req);
 
  DOMElement* element = reqs.toXML();
 
#if DEBUG
  XMLUtil::write(cout, element);
#endif
 
  reqs.fromXML(element);
  delete(element->getOwnerDocument());
 
#if DEBUG
  CPPUNIT_ASSERT(reqs.numOfRequests() == 1);
  cout << "Exiting test1! ..." << endl;
#endif
}

CppUnit::TestSuite*
CRRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("CreateResourceRequestTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<CreateResourceRequestTestCase> ( 
      "doTest", 
      &CreateResourceRequestTestCase::doTest
    )
  );

  return testSuite;
}
