#include <fstream>
#include <iostream>
#include <sstream>
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "HttpRequestTestCase.hpp"
#include "HttpRequest.hpp"
#include "InputStream.hpp"
#include "OutputStream.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace Akenti;
using namespace std;

typedef HttpRequestTestCase HRTC;

void
HRTC::doTest() {
  string method;

  try {
    method = "HttpRequestTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
HRTC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif

  HttpRequest hr;
  
  string protocol("HTTP");
  string uri("http://www.lbl.gov/lab-index/index.html");
  string method("GET");
  string contentType("text/xml");
  
  hr.setProtocol(protocol);
  hr.setURI(uri);
  hr.setMethod(method);
  hr.setContentType(contentType);

  ostringstream *oss = new ostringstream;
  OutputStream os(oss);

  hr.toStream(os);

  istringstream *iss = new istringstream(oss->str());
  InputStream is(iss);

  HttpRequest temp;

  temp.fromStream(is);

  CPPUNIT_ASSERT(protocol == hr.getProtocol());
  CPPUNIT_ASSERT(uri == hr.getURI());
  CPPUNIT_ASSERT(method == hr.getMethod());
  CPPUNIT_ASSERT(contentType == hr.getContentType());

  CPPUNIT_ASSERT(protocol == temp.getProtocol());
  CPPUNIT_ASSERT(uri == temp.getURI());
  CPPUNIT_ASSERT(method == temp.getMethod());
  CPPUNIT_ASSERT(contentType == temp.getContentType());
  CPPUNIT_ASSERT(temp.getContentLength() == hr.getContentLength());

#if DEBUG
  cout << hr.toString() << endl;
  cout << "Exiting test1" << endl;
#endif
}

CppUnit::TestSuite*
HRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("HttpRequestTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<HttpRequestTestCase> ( 
      "doTest", 
      &HttpRequestTestCase::doTest
    )
  );

  return testSuite;
}
