#include <fstream>
#include <iostream>
#include <sstream>
#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "HttpResponseTestCase.hpp"
#include "HttpResponse.hpp"
#include "InputStream.hpp"
#include "OutputStream.hpp"
#include "Exception.hpp"
#include "Error.hpp"

#ifndef DEBUG
#define DEBUG 0
#endif

using namespace Akenti;
using namespace std;

typedef HttpResponseTestCase HRTC;

void
HRTC::doTest() {
  string method;

  try {
    method = "HttpResponseTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
HRTC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif

  int statusCode = 104;
  string message("Gammasphere Status Okay");
  string protocol("HTTP");
  string contentType("text/xml");

  HttpResponse hr;

  hr.setStatusCode(statusCode);
  hr.setStatusMessage(message);
  hr.setProtocol(protocol);
  hr.setContentType(contentType);

  ostringstream *oss = new ostringstream;
  OutputStream os(oss);

  hr.toStream(os);

  istringstream *iss = new istringstream(oss->str());
  InputStream is(iss);

  HttpResponse temp;

  temp.fromStream(is);

  CPPUNIT_ASSERT(statusCode == hr.getStatusCode());
  CPPUNIT_ASSERT(message == hr.getStatusMessage());
  CPPUNIT_ASSERT(protocol == hr.getProtocol());
  CPPUNIT_ASSERT(contentType == hr.getContentType());

  CPPUNIT_ASSERT(statusCode == temp.getStatusCode());
  CPPUNIT_ASSERT(contentType == temp.getContentType());
  CPPUNIT_ASSERT(protocol == temp.getProtocol());
  CPPUNIT_ASSERT(hr.getContentLength() == temp.getContentLength());
  CPPUNIT_ASSERT(message == temp.getStatusMessage());

#if DEBUG
  cout << hr.toString() << endl;
  cout << temp.toString() << endl;
  cout << "Exiting test1" << endl;
#endif
}

CppUnit::TestSuite*
HRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("HttpResponseTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<HttpResponseTestCase> ( 
      "doTest", 
      &HttpResponseTestCase::doTest
    )
  );

  return testSuite;
}
