#include <sstream>
#include <iostream>
#include <fstream>
#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "../shared/TestCaseConstants.hpp"
#include "AttributeCertGenerator.hpp"
#include "DistinguishedName.hpp"
#include "PrivateKey.hpp"
#include "KeyFactory.hpp"
#include "Util.hpp"
#include "SecurityConstants.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"
#include "PolicyResponseTestCase.hpp"
#include "AkentiMessage.hpp"
#include "PolicyInfo.hpp"
#include "CertificateInfo.hpp"

using namespace std;

typedef PolicyResponseTestCase PRTC;

void 
PRTC::doTest() {
  string method;

  try {
    method = "PolicyResponseTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void 
PRTC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif

  PolicyInfo pi;
  CertificateInfo ci;
  vector<CertificateInfo> v(1, ci);

  pi.setCertificateInfos(v);

  DOMImplementation* impl = DOMImplementationRegistry::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = impl->createDocument(0, XMLUtil::convert("PolicyInfo"), 0);
  DOMElement* element = doc->getDocumentElement();

  pi.toXML(element);
#if DEBUG
  XMLUtil::write(cout, element);
#endif
  pi.fromXML(element);
  delete(doc);

  CPPUNIT_ASSERT(v.size() == pi.getCertificateInfos().size());
#if DEBUG
  cout << "\nExiting test1 ..." << endl;
#endif
}

CppUnit::TestSuite*
PRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("PolicyResponseTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<PolicyResponseTestCase> ( 
      "doTest", 
      &PolicyResponseTestCase::doTest
    )
  );

  return testSuite;
}

AkentiCertificate 
PRTC::genAttributeCert() {
#if DEBUG
  cout << "\n Generating Attribute Certificate ..." << endl;
#endif

  UtcTime now;
  UtcTime after(365 * 24 * 60 * 60);

  DistinguishedName ca(CAIA_DN);
  DistinguishedName issuer(SHI_DN);
  AkentiPrincipal guarantor(issuer, ca);

  DistinguishedName user(USERI_DN);
  AkentiPrincipal principal(user, ca);

  AttributeCertGenerator gen;

  gen.setVersionNumber(2);
  gen.setUniqueID("I");
  gen.setNotBefore(now);
  gen.setNotAfter(after);
  gen.setIssuer(guarantor);
  gen.setAttribute("JOB");
  gen.setValue("SUPERVISOR");
  gen.setPrincipal(principal);

  KeyFactory kfac = KeyFactory::getInstance();
  PrivateKey privKey = kfac->initPrivateKeyFromFile("PEM", ID_DIR + SHI_KEY);

  gen.sign(privKey, SecurityConstants::RSA_MD5);

  AkentiCertificate cert = gen.generate();

  return cert;
}
