#include <fstream>
#include <iostream>
#include <sstream>
#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "Exception.hpp"
#include "Error.hpp"
#include "SoapEnvelopeTestCase.hpp"
#include "SoapEnvelope.hpp"
#include "InputStream.hpp"
#include "OutputStream.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"

using namespace Akenti;
using namespace std;

typedef SoapEnvelopeTestCase SETC;

void
SETC::doTest() {
  string method;

  try {
    method = "SoapEnvelopeTestCase:doTest:test1:";
    test1();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
SETC::test1() {
#if DEBUG
  cout << "Entering test1 ..." << endl;
#endif

  DOMImplementation* impl = DOMImplementationRegistry::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = impl->createDocument(0, XMLUtil::convert("SoapEnvelope"), 0);
  DOMElement* content = doc->getDocumentElement();

  SoapEnvelope se1(content);

  ostringstream *oss1 = new ostringstream;
  OutputStream os1(oss1);

  se1.toStream(os1);

  istringstream *iss = new istringstream(oss1->str());
  InputStream is(iss);

  SoapEnvelope se2;
  
  se2.fromStream(is);

  ostringstream *oss2 = new ostringstream;
  OutputStream os2(oss2);
  se2.toStream(os2);

  CPPUNIT_ASSERT(content->isEqualNode(se1.getContent()));
  CPPUNIT_ASSERT(oss1->str() == oss2->str());
 
  const DOMElement *se2cont = se2.getContent();


  DOMElement *ele = se1.releaseContent();

  CPPUNIT_ASSERT(ele->isEqualNode(content));
  CPPUNIT_ASSERT(NULL == se1.getContent());

#if DEBUG
  cout << "Exiting test1 ..." << endl;
#endif
}

CppUnit::TestSuite*
SETC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("SoapEnvelopeTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<SoapEnvelopeTestCase> ( 
      "doTest", 
      &SoapEnvelopeTestCase::doTest
    )
  );

  return testSuite;
}
