#include <fstream>
#include <iostream>
#include <sstream>
#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "Exception.hpp"
#include "Error.hpp"
#include "SoapHttpRequestTestCase.hpp"
#include "SoapHttpRequest.hpp"
#include "InputStream.hpp"
#include "OutputStream.hpp"
#include "Xerces-Includes.hpp"
#include "XMLUtil.hpp"

using namespace Akenti;
using namespace std;

typedef SoapHttpRequestTestCase SHRTC;

void
SHRTC::doTest() {
  string method;

  try {
    method = "SoapHttpRequestTestCase:doTest:testRequestFromSetters:";
    testRequestFromSetters();
  } catch (Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch (Error& err) {
    CppUnit::Message m(method + err.getMessage());

    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void
SHRTC::testRequestFromSetters() {
#if DEBUG
  cout << "Entering testRequestFromSetters ..." << endl;
#endif

  SoapHttpRequest shr;

  string protocol("HTTP");
  string uri("http://www.lbl.gov/lab-index/index.html");
  string method("POST");
  string contentType("text/xml");
  
  shr.setProtocol(protocol);
  shr.setURI(uri);
  shr.setMethod(method);
  shr.setContentType(contentType);

  DOMImplementation* impl = DOMImplementationRegistry::getDOMImplementation(XMLUtil::convert("Core"));
  DOMDocument* doc = impl->createDocument(0, XMLUtil::convert("SoapHttqRequest") , 0);
  DOMElement* params = doc->getDocumentElement();

  shr.setParams(params);

  ostringstream *oss = new ostringstream;
  OutputStream os(oss);

  shr.toStream(os);

  istringstream *iss = new istringstream(oss->str());
  InputStream is(iss);

  SoapHttpRequest temp;

  temp.fromStream(is);

  ostringstream *oss2 = new ostringstream;
  OutputStream os2(oss2);

  temp.toStream(os2);

#if DEBUG
  cout << "SoapRequest1:\n" << oss->str() << endl;
  cout << "SoapRequest2:\n" << oss2->str() << endl;  
#endif

  CPPUNIT_ASSERT(protocol == shr.getProtocol());
  CPPUNIT_ASSERT(uri == shr.getURI());
  CPPUNIT_ASSERT(method == shr.getMethod());
  CPPUNIT_ASSERT(contentType == shr.getContentType());
  CPPUNIT_ASSERT(params->isEqualNode(shr.getParams()));

  CPPUNIT_ASSERT(protocol == temp.getProtocol());
  CPPUNIT_ASSERT(uri == temp.getURI());
  CPPUNIT_ASSERT(method == temp.getMethod());
  CPPUNIT_ASSERT(contentType == temp.getContentType());
  CPPUNIT_ASSERT(oss->str() == oss2->str());


  const DOMElement *tp = temp.getParams();

  string str3(XMLUtil::convert(params->getLocalName()));
  string str4(XMLUtil::convert(tp->getLocalName()));

#if DEBUG
  cout << "\nExiting testRequestFromSetters ..." << endl;
#endif
}

CppUnit::TestSuite*
SHRTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("SoapHttpRequestTestCase");

  testSuite->addTest (
    new CppUnit::TestCaller<SoapHttpRequestTestCase> ( 
      "doTest", 
      &SoapHttpRequestTestCase::doTest
    )
  );

  return testSuite;
}
