#include <iostream>
#include "cppunit/TextTestRunner.h"
#include "Provider.hpp"
#include "OpenSSLProviderImpl.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "XMLUtil.hpp"
#include "CheckAccessRequestTestCase.hpp"
#include "CheckAccessResponseTestCase.hpp"
#include "HttpRequestTestCase.hpp"
#include "HttpResponseTestCase.hpp"
#include "SoapEnvelopeTestCase.hpp"
#include "SoapHttpRequestTestCase.hpp"
#include "SoapHttpResponseTestCase.hpp"
#include "CertificateResponseTestCase.hpp"
#include "PolicyResponseTestCase.hpp"
#include "CheckPolicyRequestTestCase.hpp"
#include "CheckPolicyResponseTestCase.hpp"
#include "ResourceBrowseResponseTestCase.hpp"
#include "ResourceBrowseRequestTestCase.hpp"
#include "CreateResourceRequestTestCase.hpp"
#include "CreateResourceResponseTestCase.hpp"
#include "UploadCertificateRequestTestCase.hpp"
#include "UploadCertificateResponseTestCase.hpp"
#include "DeleteCertificateRequestTestCase.hpp"
#include "DeleteCertificateResponseTestCase.hpp"
#include "GetResourceInfoRequestTestCase.hpp"
#include "GetResourceInfoResponseTestCase.hpp"

using namespace Akenti;
using namespace std;

int
main(int ac, char **argv) {
  try {
    Provider po = Provider(new OpenSSLProviderImpl);
    
    po->loadDigestAlgorithms();
    po->loadSignatureAlgorithms();

    Provider::setProvider(po);
    XMLUtil::init();

    CppUnit::TextTestRunner runner;

    runner.addTest(HttpRequestTestCase::suite());
    runner.addTest(HttpResponseTestCase::suite());
    runner.addTest(SoapEnvelopeTestCase::suite());  
    runner.addTest(SoapHttpRequestTestCase::suite());
    runner.addTest(SoapHttpResponseTestCase::suite());  

    runner.addTest(CheckAccessRequestTestCase::suite());  
    runner.addTest(CheckAccessResponseTestCase::suite());

    runner.addTest(CollectPolicyRequestTestCase::suite());
    runner.addTest(ResourceBrowseRequestTestCase::suite());
    runner.addTest(ResourceBrowseResponseTestCase::suite());
    runner.addTest(CreateResourceRequestTestCase::suite());
    runner.addTest(CreateResourceResponseTestCase::suite());

    runner.addTest(CertificateResponseTestCase::suite());
    runner.addTest(PolicyResponseTestCase::suite());

    runner.addTest(CheckPolicyResponseTestCase::suite());
    runner.addTest(UploadCertificateRequestTestCase::suite());
    runner.addTest(UploadCertificateResponseTestCase::suite());

    runner.addTest(DeleteCertificateRequestTestCase::suite());
    runner.addTest(DeleteCertificateResponseTestCase::suite());

    runner.addTest(GetResourceInfoRequestTestCase::suite());
    runner.addTest(GetResourceInfoResponseTestCase::suite());

    runner.run("", false);
    return 0;
  } catch (Exception& ex) {
    cout << "TestServerClient:main:Exception" + ex.getMessage() << endl;
  } catch (Error& err) {
    cout << "TestServerClient:main:FATAL" + err.getMessage() << endl;
  }

  return 1;
}
