#include <fstream> 
#include "cppunit/TestCaller.h"
#include "CreateAkentiConf.hpp"
#include "TestCaseConstants.hpp"
#include "AkentiConstants.hpp"
#include "File.hpp"

typedef CreateAkentiConf CAC;

using Akenti::File;

void
CAC::createAkentiConfig() {
  string akentiConfFile = CONF_DIR + CONF_FILE;

  std::ofstream os(akentiConfFile.c_str());

  os << "# Akenti server may be providing access \n"
     << "# control to several independent resources. \n"
     << "# So for each top level resource we must \n"
     << "# specify a base which must be an absolute path. \n"
     << "RootResourceName " << ROOT1 << " " << File(BASE_DIR).getAbsolutePath() << "\n"   
     << "RootResourceName " << ROOT2 << " " << File(BASE_DIR).getAbsolutePath() << "\n"   
     << "RootResourceName " << ROOT3 << " " << File(BASE_DIR).getAbsolutePath() << "\n"   
     << "RootResourceName " << ROOT4 << " " << File(BASE_DIR).getAbsolutePath() << "\n"   
     << "# \n" 
     << "# \n" 
     << "# The name of policy files.\n"	   
     << "# \n" 
     << "PolicyFileName " << Akenti::PolicyFileName << "\n"
     << "# \n" 
     << "# The name of hint files used by Java GUI generators. \n"	   
     << "ResAttrFileName" << Akenti::ResAttrFileName << "\n"
     << "# \n" 
     << "# \n"
     << "# The encoding type  of Akenti Certificates (XML or PEM).\n"	   
     << "# Leave blank to support both.\n"
     << "# \n" 
     << "AK_CERT_ENC_TYPE " << AK_CERT_ENC_TYPE << '\n' 
     << "# \n" 
     << "#############################################################\n"
     << "#                                                           #\n"
     << "#                       BEGIN CACHE CONFIG                  #\n"
     << "#                                                           #\n"
     << "# Caching Parameters:                                       #\n"
     << "#                                                           #\n"
     << "# Caching   [on, off]                                       #\n"
     << "# CacheDir  the name of the caching directory               #\n"
     << "#                                                           #\n"
     << "Caching   " << "on" << "\n"
     << "CacheDir " << File(CACHE_DIR).getAbsolutePath() << "\n"
     << "#                                                           #\n"
     << "#                      END CACHE CONFIG                     #\n"
     << "#                                                           #\n"
     << "#############################################################\n"
     << "\n"
     << "#############################################################\n"
     << "#                                                           #\n"
     << "#                    BEGIN AUTH SERVER INFO                 #\n"
     << "#                                                           #\n"
     << "#    SSLMode   [on, off]  Use SSL                           #\n"
     << "#    AkentiPort       default secure port 8443              #\n"
     << "#                     default unsecure port 8444            #\n"
     << "#    ServerCertFile   server's certificate pathanme         #\n"
     << "#    ServerKeyFile    server's private key pathname         #\n"
     << "#                                                           #\n"
     << "#                                                           #\n"
     << "SSLMode        " << "on" << "\n"
     << "ServerCertFile " << File(ID_DIR + SERVER_CERT).getAbsolutePath() << "\n"
     << "ServerKeyFile  " << File(ID_DIR + SERVER_KEY).getAbsolutePath()  << "\n"
     << "#                                                           #\n"
     << "#    SignCerts [on, off] Signing capability and cached certs. \n"
     << "#                                                           #\n"
     << "SignCerts  on \n"
     << "# \n" 
     << "#                                                           #\n"
     << "#                    END AUTH SERVER INFO                   #\n"
     << "#                                                           #\n"
     << "#############################################################\n"
     << "\n"
     << "#############################################################\n"
     << "#                                                           #\n"
     << "#              BEGIN LOGGING CONFIG                         #\n"
     << "# Logging parameters:                                       #\n"
     << "#                                                           #\n"
     << "# Type of logging desired                                   #\n"
     << "# Logging  [off, file, server, stdout, stderr]              #\n"
     << "#                                                           #\n"
     << "# Setting the logging level                                 #\n"
     << "#   LogLevel [1, 2, 3, 4, 5]                                #\n"
     << "#   Only logs with a level <= LogLevel will be logged       #\n"
     << "#                                                           #\n"
     << "# Logging to a file:                                        #\n"
     << "#   TempLogDir  temporary storage                           #\n"
     << "#   MonitorLogFile permanent storage                        #\n"
     << "#                                                           #\n"
     << "# Logging to a server                                       #\n"
     << "#   MonitorServer   server's hostname                       #\n"
     << "#   MonitorPort     server's port                           #\n"
     << "#                                                           #\n"
     << "Logging  " << "file" << "\n"
     << "LogLevel " <<  2 << "\n"
     << "# File Logging Mechanism\n"
     << "TempLogDir     " << "/tmp" << "\n"
     << "MonitorLogFile " << File(LOG_DIR + LOG_FILE).getAbsolutePath() << "\n"
     << "# Server Logging Mechanism\n"
     << "MonitorServer  " << "\n" 
     << "MonitorPort    " << "\n"
     << "#                                                           #\n"
     << "#              END LOGGING CONFIG                           #\n"
     << "#                                                           #\n"
     << "#############################################################\n";

  os.close();
  CPPUNIT_ASSERT(!os.fail());
}
  
void
CAC::createClientConfig() {
  string akentiConfFile = CONF_DIR + CLIENT_CONF_FILE;

  std::ofstream os(akentiConfFile.c_str());

  os << "#############################################################\n"
     << "#                                                           #\n"
     << "#                                                           #\n"
     << "#                    BEGIN CLIENT'S CREDENTIALS             #\n"
     << "#    SSLMode   [on, off]  Use SSL                           #\n"
     << "#    Host             default localhost                     #\n"
     << "#    Port             default secure port 8443              #\n"
     << "#                     default unsecure port 8444            #\n"
     << "#    ClientCertFile   client's certificate pathanme         #\n"
     << "#    ClientKeyFile    client's private key pathname         #\n"
     << "#    CertificateAuthorityPath  dirpath to trusted CAS       #\n"
     << "#    CertificateAuthorityFile  file containing trusted CA   #\n"
     << "#                                                           #\n"
     << "SSLMode        " << "on" << "\n"
     << "ClientCertFile " << File(ID_DIR + CLIENT_CERT).getAbsolutePath() << "\n"
     << "ClientKeyFile  " << File(ID_DIR + CLIENT_KEY).getAbsolutePath()  << "\n"
     << "CertificateAuthorityPath " << "\n"
     << "CertificateAuthorityFile " << File(ID_DIR + CAI_CERT).getAbsolutePath() << "\n"
     << "#                                                           #\n"
     << "#                    END CLIENT'S CREDENTIALS               #\n"
     << "#                                                           #\n"
     << "#############################################################\n"
     << "\n";
  os.close();
  CPPUNIT_ASSERT(!os.fail());
}

CppUnit::TestSuite*
CAC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("CreateAkentiConf");

  testSuite->addTest(new CppUnit::TestCaller<CreateAkentiConf>(
    "createAkentiConfig", &CreateAkentiConf::createAkentiConfig)
  );

  testSuite->addTest(new CppUnit::TestCaller<CreateAkentiConf>(
    "createClientConfig", &CreateAkentiConf::createClientConfig)
  );

  return testSuite;
}
