#include "cppunit/Exception.h"
#include "cppunit/TestCaller.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "InstallResourceTree.hpp"
#include "TestCaseConstants.hpp"
#include "File.hpp"
#include "Exception.hpp"
#include "Error.hpp"

typedef InstallResourceTree IRT;

using namespace std;
using namespace Akenti;

void 
IRT::createResourceTree() {
#if DEBUG
  cout << "Begin creating resource tree" << endl;
#endif

  string method = "InstallResourceTree:createResourceTree:";

  try {
    createDir(BASE_DIR);
    createDir(LOG_DIR);
    createDir(ID_DIR); 
    createDir(AKCERT_DIR);
    createDir(CONF_DIR);
    createDir(CACHE_DIR);
    createDir(ROOT1_DIR);
    createDir(ROOT2_DIR);
    createDir(SUB1_DIR);
    createDir(SUB2_DIR);
    createDir(SUB3_DIR);
    createDir(SUB4_DIR);
    createDir(ROOT3_DIR);
    createDir(ROOT4_DIR);
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }  

#if DEBUG
  cout << "End creating resource tree" << endl;
#endif

}

void
IRT::createDir(const string& dirName) {
  Akenti::File dir(dirName);
  
  if (dir.exists()) {    
    return;
  } 

  dir.mkdir();
}

CppUnit::TestSuite*
IRT::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("InstallResourceTree");

  testSuite->addTest(
    new CppUnit::TestCaller<InstallResourceTree>(
      "createResourceTree",
      &InstallResourceTree::createResourceTree
    )
  );

  return testSuite;
}

