#include <sstream>
#include <cppunit/Exception.h>
#include "cppunit/TestCaller.h"
#include "cppunit/TextTestRunner.h"
#include "cppunit/Message.h"
#include "cppunit/SourceLine.h"
#include "StreamTokenizerTestCase.hpp"
#include "StreamTokenizer.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace Akenti;
using namespace std;

typedef StreamTokenizerTestCase STTC;

void
STTC::doTest() {
  string method;

  try {
    method = "StreamTokenizerTestCase:test1:";
    test1();
    method = "StreamTokenizerTestCase:test2:";
    test2();
  } catch(Exception& ex) {
    CppUnit::Message m(method + ex.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  } catch(Error& err) {
    CppUnit::Message m(method + err.getMessage());
 
    throw CppUnit::Exception(m, CPPUNIT_SOURCELINE());
  }
}

void 
STTC::test1() {
  istringstream s("/CN= Willie        ");
  StreamTokenizer st(s, "/= ", false);

  CPPUNIT_ASSERT(st.nextToken() == "CN");
  CPPUNIT_ASSERT(st.nextToken() == "Willie");
  CPPUNIT_ASSERT(st.hasMoreTokens() == false);
}

void 
STTC::test2() {
  istringstream s("/CN=Willie");
  StreamTokenizer st(s, "/=", true);

  CPPUNIT_ASSERT(st.nextToken() == "/");
  CPPUNIT_ASSERT(st.nextToken() == "CN");
  CPPUNIT_ASSERT(st.nextToken() == "=");
  CPPUNIT_ASSERT(st.nextToken() == "Willie");
  CPPUNIT_ASSERT(st.hasMoreTokens() == false);
}

CppUnit::TestSuite*
STTC::suite() {
  CppUnit::TestSuite* testSuite = new CppUnit::TestSuite("StreamTokenizerTestCase");

  testSuite->addTest(
    new CppUnit::TestCaller<StreamTokenizerTestCase>(
      "doTest",
      &StreamTokenizerTestCase::doTest
    )
  );

  return testSuite;
}
