#ifndef X509_CERT_TEST_CASE_H   
#define X509_CERT_TEST_CASE_H 

#include "cppunit/TestSuite.h"
#include "cppunit/TestCase.h"
#include "PrivateKey.hpp"
#include "PublicKey.hpp"
#include "DistinguishedName.hpp"
#include "Certificate.hpp"

/**
  * Generates <i>x509</i> certificates for the Akenti test.
  * 
  * @version  1.1 00/05/02 
  * @author Willie Chin 
  * @author Abdelilah Essiari
  */ 
class X509CertTestCase : public CppUnit::TestCase {

public:
  
  /**
    * Constructs this object.
    */  
  X509CertTestCase(const std::string& name = "X509CertTestCase") 
    : TestCase(name) {
  }   

  /**
    * Register the test methods.
    */
  static CppUnit::TestSuite* suite();

private:

  /**
    * Generates all the X509 certificates. 
    */
  void generateX509Certificates();

  /**
    * Generates CAI's x509 certificate.
    */
  void generateCAICert();

  /**
    * Generates CAIA's x509 certificate.
    */
  void generateCAIACert();

  /**
    * Generates UserI's x509 certificate.
    */
  void generateUserICert();


  /**
    * Generates StakeHolderI's  certificate.
    */
  void generateSHICert();

  /**
    * Generates server's certificate.
    */
  void generateServerCert();

  /**
    * Generates client's certificate.
    */
  void generateClientCert();

  /**
    * Generates proxy's certificate.
    */
  void generateProxyCert(); 

  /**
    * Helper function.
    */
  Akenti::Certificate 
  generateCertificate(const Akenti::PrivateKey& privKey,
                      const Akenti::DistinguishedName& issuer,
                      const Akenti::PublicKey& pubKey,
                      const Akenti::DistinguishedName& subject);
}; 

#endif
