#include <iostream>
#include <string>
#include <unistd.h>
#include "Provider.hpp"
#include "OpenSSLProviderImpl.hpp"
#include "AkentiCertificate.hpp"
#include "XMLUtil.hpp"
#include "File.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace std;
using namespace Akenti;

/**
  * <pre>
  * <b>ConvertAkentiCert:</b>
  * Use this program to convert an akenti certificate from/to PEM/XML formats.
  * </pre> 
  */
namespace ConvertAkentiCert {

void 
printPurpose() {
  cerr << "Use this program to convert an akenti certificate"  << endl
       << "from one format to another. " << endl
       << endl
       << "Currently we support two formats PEM and XML" << endl
       << endl;
}

void 
printUsage() {
  cerr << "Usage: ConvertAkentiCert args " << endl
       << "  -a arg      - akenti certificate" << endl
       << "  [-i arg]    - in format PEM or XML" << endl
       << "  [-o arg]    - out format PEM or XML" << endl
       << "  [-v]        - verbose " << endl 
       << "  [-h]        - prints this message" << endl; 
}

int
main(int argc, char *argv[]) {
  string inFile;
  string inFormat;
  string outFormat;
  int opt;
  bool verbose = false;

  if (argc < 2) {
    printPurpose();
    printUsage();
    exit(2);
  }

  while ((opt = getopt(argc,argv,"a:i:o:vh")) != EOF) {
    switch(opt){
      case 'a':
        inFile = optarg;
        break;
      case 'i':
        inFormat = optarg; 
        break;
      case 'o':
        outFormat = optarg;
        break;
      case 'v':
        verbose = true;
        break;
      case 'h':
      default:
        printUsage();
        exit(2);
    }
  }

  try {
    if (verbose) {
      cout << "Initializing XML Library ..." << endl;
    }

    XMLUtil::init();

    if (verbose) {
      cout << "Setting security provider ..." << endl;
    }

    Provider po = Provider(new OpenSSLProviderImpl);
    
    po->loadDigestAlgorithms();
    po->loadSignatureAlgorithms();
    Provider::setProvider(po);

    // Read in Akenti certificate and print the results
    if (verbose) {
      cout << "Reading Akenti Certificate from " << inFile << endl;
    }

    File file(inFile);

    if (!file.isFile()) {
      cerr << "Cant open file at " << file.getAbsolutePath() << endl;
      exit(2);
    }

    AkentiCertificate cert = AkentiCertificate::initFromFile(inFormat, inFile);
    AkentiCertificate::writeToStream(outFormat, cout, cert);
    exit(0);
  } catch (Exception& ex) {
    cerr << "Caught Exception" << ex.getMessage() << endl;
    exit(3);
  } catch (Error& err) {
    cerr << "Caught Error (Report to Akenti Team)" << err.getMessage() <<endl;
    exit(4);
  }
}

}
