#include <sstream>
#include "AKCertHolder.hpp"
#include "CertificateStatusListener.hpp"

using std::ostringstream;

namespace Akenti {

typedef AKCertHolderImpl ACHI;

ACHI::AKCertHolderImpl(const AkentiCertificate& cert, 
                       int level,
                       const UtcTime& startCacheTime,
                       const UtcTime& endCacheTime) 
  : cert(cert),
    level(level),
    startCacheTime(startCacheTime),
    endCacheTime(endCacheTime) {
}

void 
ACHI::addCertificateStatusListener(CertificateStatusListener* l) {
  this->statusListeners.push_back(l);
}

void 
ACHI::removeCertificateStatusListener(CertificateStatusListener* csl) {

  // Remove all status listener objects with address csl and save
  // iterator to the new "end" of the vector.
  vector<CertificateStatusListener*>::iterator new_end =
    remove(statusListeners.begin(), statusListeners.end(), csl);

  // Check if anything was removed
  if(new_end == statusListeners.end())
    throw Error("AKCertHolderImpl:removeCertificateStatusListener:No such listener");

  // Shorten the vector to the proper length, completing the removal
  statusListeners.erase(new_end, statusListeners.end());
}

const Akenti::AkentiCertificate& 
ACHI::getCertificate() const {
  return cert;
}

bool
ACHI::setLevel(int level) {
  if (level < 0) {
    vector<CertificateStatusListener*> temp = statusListeners;

    this->level = level;

    for (int i = 0; i < temp.size(); ++i) {
      temp[i]->verifyingCertFailed(cert);
    }

    return true;
  } else if (this->level < level) {
    this->level = level;

    for (int i = 0; i < statusListeners.size(); ++i) {
      statusListeners[i]->verifyingCertOK(cert);
    }
   
    return true;
  }

  return false;
}
  
int 
ACHI::getLevel() const {
  return level;
}

bool
ACHI::setStartCacheTime(const UtcTime& startCacheTime) {
  if (this->startCacheTime > startCacheTime) {
    this->startCacheTime = startCacheTime;

    for (int i = 0; i < statusListeners.size(); ++i) {
      statusListeners[i]->certStartCacheTimeChanged(cert);   
    }
    
    return true;
  }
  
  return false;
}

const Akenti::UtcTime& 
ACHI::getStartCacheTime() const {
  return startCacheTime;
}

bool
ACHI::setEndCacheTime(const UtcTime& endCacheTime) {
  if (this->endCacheTime < endCacheTime) {
    this->endCacheTime = endCacheTime;

    for (int i = 0; i < statusListeners.size(); ++i) {
      statusListeners[i]->certEndCacheTimeChanged(cert);   
    }
    
    return true;
  }
  
  return false;
}

const Akenti::UtcTime& 
ACHI::getEndCacheTime() const {
  return endCacheTime;
}

bool
ACHI::checkValidity(const UtcTime& utc) {
  return startCacheTime <= utc && utc <= endCacheTime && cert->checkValidity(utc);
}

string 
ACHI::paramString() const {
  ostringstream os;

  os << '['
     << getCertificate()->paramString()
     << ',' 
     << getLevel()
     << ',' 
     << getStartCacheTime().paramString()
     << ',' 
     << getEndCacheTime().paramString()
     << ']';
        
  return os.str();
}

}
