#include "AkentiPrincipal.hpp"
#include "AkentiOutputStream.hpp"
#include "AkentiInputStream.hpp"
#include "XMLUtil.hpp"
#include "Xerces-Includes.hpp"

namespace Akenti {

typedef AkentiPrincipal AP;
typedef DistinguishedName DN; 

AP::AkentiPrincipal() : name(), guarantor() {
}

AP::AkentiPrincipal(const DN& name, const DN& guarantor)
  : name(name), guarantor(guarantor) {
}

AP::AkentiPrincipal(const string& name, const string& guarantor) 
  : name(name), guarantor(guarantor) {
}

AP::~AkentiPrincipal() {
}

void 
AP::operator=(const AkentiPrincipal& ap) {
  name = ap.name;
  guarantor = ap.guarantor;
}

const DN&
AP::getName() const {
  return name;
}

const DN&
AP::getGuarantor() const {
  return guarantor;
}

bool 
AP::operator==(const AkentiPrincipal& ap) const {
  return name == ap.getName() && guarantor == ap.getGuarantor();
}

void
AP::writeObject(AkentiOutputStream& drain) const {
  drain <<  name.toString()
        << ' '
        <<  guarantor.toString();
}

void 
AP::readObject(AkentiInputStream& source) {
  string temp;

  source >> temp;
  name = temp; 
  source >> temp;
  guarantor = temp; 
}

void 
AP::toXML(DOMElement* ap) const {
  DOMDocument *doc = ap->getOwnerDocument();
  DOMElement *userDN = doc->createElement(XMLUtil::convert("UserDN"));
  DOMElement *caDN = doc->createElement(XMLUtil::convert("CADN"));

  DOMNode *node1 = (DOMNode *) doc->createTextNode(XMLUtil::convert(name.toString().c_str()));
  DOMNode *node2 = (DOMNode *) doc->createTextNode(XMLUtil::convert(guarantor.toString().c_str()));
  userDN->appendChild(node1);
  caDN->appendChild(node2);
  ap->appendChild(userDN);
  ap->appendChild(caDN);
}

void 
AP::fromXML(const DOMElement* ap) {
  DOMNode *temp = ap->getFirstChild();
  DOMElement *userDN = (DOMElement *) temp;
  
  temp = userDN->getFirstChild();
  name = XMLUtil::convert(temp->getNodeValue());
  temp = userDN->getNextSibling();

  DOMElement *caDN = (DOMElement *) temp;
  temp = caDN->getFirstChild();
  guarantor = XMLUtil::convert(temp->getNodeValue());
}

string
AP::paramString() const {
  string params;

  params = '[';
  params += "name";
  params += ':';
  params += getName().toString();
  params += ',';
  params += "ca";
  params += ':';
  params += getGuarantor().toString();
  return params += ']';
}

string
AP::toLogString() const {
  string log;

  log += "PRINCIPAL_";
  log += getName().toLogString();
  log +=  ' ';
  log += "CA_";
  log += getGuarantor().toLogString();
  return log;
}

} // end of namaspace Akenti




