#include "Directory.hpp"
#include "AkentiOutputStream.hpp"
#include "AkentiInputStream.hpp"
#include "URL.hpp"
#include "Exception.hpp"
#include "Error.hpp"
#include "XMLUtil.hpp"
#include "Xerces-Includes.hpp"
#include "Util.hpp"

namespace Akenti {

typedef Directory DIR;

  //DIR::Directory() : url(URL().toString()) {
  //}

DIR::Directory() : url("noprot://nohost") {
}
  
DIR::Directory(const string& url) : url(url) {
  if (!url.size()) {
    throw Error("Directory:constructor:illegal arg");
  }
}
  
DIR::~Directory() {
}

void 
DIR::operator=(const Directory& dir) {
  url = dir.url;
}

const string&
DIR::toString() const {
  return url;
}

bool
DIR::operator==(const Directory& dir) const {
  return url == dir.url;
}

bool
DIR::parseDir(URL& temp) const {
  try {
    if (url[url.size() - 1] != '/') {
      temp = URL(url + '/');
    } else {
      temp = URL(url);
    }
    return true;
  } catch(Exception& ex) {
    return false;
  }
}

void 
DIR::writeObject(AkentiOutputStream& drain) const {
  drain << url;
}

void 
DIR::readObject(AkentiInputStream& source) {
  source >> url;
}

void 
DIR::toXML(DOMElement *dir) const {
  DOMDocument* doc = dir->getOwnerDocument();
   
  dir->appendChild(doc->createTextNode(XMLUtil::convert(url.c_str())));
}

void 
DIR::fromXML(const DOMElement *dir) {
  DOMNode *temp = dir->getFirstChild();

  if (temp == NULL) {
    throw Exception("Directory:fromXML:missing url");
  }

  url = XMLUtil::convert(temp->getNodeValue());
}

string 
DIR::paramString() const {
  string params;

  params += "url";
  params += '=';
  params += '"';
  params += url;
  params += '"';
  return params;
}

} 
