#include <cstdio> 
#include <sstream>
#include "IOHandler.hpp"

using std::ostringstream;

extern "C" {
  int close(int);
  ssize_t read(int,  void*, size_t);
  ssize_t write(int, const void*, unsigned int);
}

typedef Akenti::IOHandlerImpl IOHI;

IOHI::IOHandlerImpl(int fd) : fd(fd) {
}
  
IOHI::~IOHandlerImpl() {
  close();
}

int 
IOHI::getDescriptor() const {
  return fd;
}

bool 
IOHI ::isOpen() const {
  return fd != -1;
}

bool 
IOHI::isClosed() const {
  return fd == -1;
}

int 
IOHI::write(const char* buff, int num) {
  return ::write(fd, (void *) buff, num);
}

int 
IOHI::read(char* buff, int num) {
  return ::read(fd, (void *) buff, (size_t) num);
}

int 
IOHI::close() {
  return fd < 0 ? -1 : (::close(fd), fd = -1, 0);
}

string
IOHI::paramString() const {
  ostringstream os;

  os << '['
     << "fd"
     << '='
     << getDescriptor()
     << ']';
  return os.str();
}
