#ifndef AK_SOCKET_IN_BUFFER_HPP
#define AK_SOCKET_IN_BUFFER_HPP

#if defined __GNUC__ && __GNUC__ < 3 
# include <streambuf.h>
#else 
# include <streambuf>
#endif

#include "IOHandler.hpp"

using std::streambuf;

/**
  * A <i>SocketInBuffer<i> is a particular type of stream buffer for 
  * sockets in the input buffer.
  * 
  * @see IOHandler
  *
  * @version 1.1 00/11/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

namespace Akenti {

const int PUSHBACK_SIZE = 16;
const int SOCKBUFF_SIZE = 1024;

class SocketInBuffer : public streambuf {

public:

  /**
    * Constructs this object.
    * 
    * @param fd the descriptor
    */
  SocketInBuffer(const IOHandler& fd);

  /**
    * Destroys this object.
    */  
  ~SocketInBuffer();

  /**
    * Returns a string representation for SocketInBuffer which is used for  
    * debugging purposes.
    *
    * @return debugging info
    */
  string paramString() const;

protected:

  /**
    * Reponsible for inserting characters into the buffer.
    *
    * @return the current read position of the buffer 
    */
  int underflow();

private:

  IOHandler fd;

  char buffer[SOCKBUFF_SIZE];

};

} // end of namespace Akenti

#endif
