#ifndef AK_SOCKET_OUT_BUFFER_HPP
#define AK_SOCKET_OUT_BUFFER_HPP

#if defined __GNUC__ && __GNUC__ < 3  
# include <streambuf.h> 
#else  
# include <streambuf> 
#endif

#include "IOHandler.hpp" 

using std::streamsize;
using std::streambuf;

/**
  * A <i>SocketOutBuffer</i> is a user-defined stream buffer for writing
  * characters into the stream. 
  *
  * @version 1.1 00/11/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

namespace Akenti {

class SocketOutBuffer : public streambuf {

public:

  /** 
    * Constructs this object
    */  
  SocketOutBuffer(const IOHandler& fd);

  /**
    * Destroys this object 
    */
  ~SocketOutBuffer();

  /**
    * Returns a string representation for SocketOutBuffer which is used for 
    * debugging purposes.
    *
    * @return debugging info
    */
  string paramString() const;    

protected:

  /**
    * Reponsible for sending the characters out from the buffer to the 
    * output device
    * 
    * @param c the integer to be written 
    * @return EOF value of the the integer that has been written 
    */
  int overflow(int c);

  /**
    * Writes the whole character sequence with one call to the 
    * descriptor fd
    *
    * @param s the character string
    * @param num the size of the character string
    * @return streamsize the size of characters written
    */
  streamsize xsputn(const char *s, streamsize num);

private:

  IOHandler fd;

};

} // end of namespace Akenti

#endif
