#ifndef OPEN_SSL_X509_REVOKED_CERTIFICATE_IMPL_HPP
#define OPEN_SSL_X509_REVOKED_CERTIFICATE_IMPL_HPP

#include "RevokedCertificateImpl.hpp"

namespace Akenti {

class Certificate;

/**
  * This class is the subclass of <i>RevokedCertificateImpl</i>
  * it implements Certificate following the X.509 standard.
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */
class OpenSSLX509RevokedCertificateImpl : public RevokedCertificateImpl {

public:

  /**
    * Constructs this object with arguments cert and revokedDate.
    */ 
  OpenSSLX509RevokedCertificateImpl(const Certificate& cert,
				    const UtcTime& revokedDate);

  /**
    * Constructs this object with arguments serialNum and revokedDate. 
    */
  OpenSSLX509RevokedCertificateImpl(long serialNum, 
				    const UtcTime& revokedDate);

  /**
    * Constructs this object with arguments serialNum, revokedDate,
    * and extensions.
    */
  OpenSSLX509RevokedCertificateImpl(
	long serialNum, 
        const UtcTime& revokedDate,
        const std::vector<V3Extension>& v3extensions);

  /**
    * Deletes this object
    */
  ~OpenSSLX509RevokedCertificateImpl();

  /** 
    * Returns the certificateNumber of the revoked certificate.
    */
  long getCertificateNumber() const;

  /**
    * Return UtcTime of the revocation.
    */ 
  UtcTime getRevocationDate() const;

  std::vector<V3Extension> getExtensions() const;

  /**
    * Returns a string representation for OpenSSLX509RevokedCertificateImpl
    * which is used for debugging purposes.
    *
    * @return debugging info
    */
  string paramString() const;

private:

  long serialno;

  UtcTime revokedDate;

  std::vector<V3Extension> v3extensions;

};

} // end namespace Akenti

#endif
